<?php

namespace DeutschePost\Internetmarke\Abstracts;

use DeutschePost\Internetmarke\Enums\HttpDataType;

defined('ABSPATH') || exit;

abstract class RestCurl
{
    private $curl;
    private $header;
    private $options;
    private $lastRequest;

    public function __construct()
    {
        // is cURL installed?
        if (!function_exists('curl_init')){
            throw new \Exception("cURL is not installed on the system.");
        }

        $this->header = array();
        $this->initDefaultHeader();

        $this->options = array();
        $this->initDefaultOptions();
    }

    protected function initCurl()
    {
        $this->curl = curl_init();
    }

    protected function closeCurl()
    {
        curl_close($this->curl);
    }

    protected function initDefaultHeader()
    {
        $this->addHeader("cache-control: no-cache");
    }

    protected function getHeader()
    {
        return $this->header;
    }

    protected function addHeader($entry)
    {
        if ($entry == null) {
            return false;
        }

        array_push($this->header, $entry);

        return true;
    }

    protected function removeHeader($entry)
    {
        if ($entry == null) {
            return false;
        }

        $position = null;

        for ($i=0; $i<count($this->header); $i++) {
            $header_entry = $this->header[$i];
            if ($header_entry === $entry) {
                $position = $i;
                break;
            }
        }

        if ($position) {
            unset($this->header[$position]);
            return true;
        }

        return false;
    }

    protected function removeHeaderAtPosition($position)
    {
        if ($position < 0 || $position == null) {
            return false;
        }

        if ($position) {
            unset($this->header[$position]);
            return true;
        }

        return false;
    }

    protected function clearHeader()
    {
        $this->header = array();
    }

    protected function initDefaultOptions()
    {
        $this->addOption(CURLOPT_HEADER, false);
        $this->addOption(CURLOPT_VERBOSE, true);
        $this->addOption(CURLOPT_RETURNTRANSFER, true);
        $this->addOption(CURLOPT_FOLLOWLOCATION, true);
        $this->addOption(CURLOPT_TIMEOUT, 10);
        $this->addOption(CURLOPT_SSL_VERIFYPEER, false);
        $this->addOption(CURLOPT_SSL_VERIFYHOST, false);
        $this->addOption(CURLINFO_HEADER_OUT, true);
    }

    protected function getOptions()
    {
        return $this->options;
    }

    protected function addOption($key, $value)
    {
        if ($key === null || $value === null) {
            return false;
        }

        $this->options[$key] = $value;

        return true;
    }

    protected function removeOption($key)
    {
        if ($key === null) {
            return false;
        }

        unset($this->options[$key]);

        return true;
    }

    protected function clearOptions()
    {
        $this->options = array();
    }

    protected function prepareHeader()
    {
        curl_setopt($this->curl, CURLOPT_HTTPHEADER, $this->header);
    }

    protected function prepareOptions()
    {
        foreach ($this->options as $key => $value)
        {
            curl_setopt($this->curl, $key, $value);
        }
    }

    protected function prepareLastRequest()
    {
        $this->lastRequest = curl_getinfo($this->curl);
    }

    protected function addData($data, $data_type)
    {
        if ($data === null || $data_type === null)
        {
            return false;
        }

        if ($data_type == HttpDataType::JSON)
        {
            $payload = json_encode($data);
            $this->addOption(CURLOPT_POSTFIELDS, $payload);
            $this->addHeader("Content-Type: application/json");
        }
        elseif ($data_type == HttpDataType::URL_ENCODED)
        {
            $payload = http_build_query($data, null, null, PHP_QUERY_RFC1738);
            $this->addOption(CURLOPT_POSTFIELDS, $payload);
            $this->addHeader("Content-Type: application/x-www-form-urlencoded");
        }
        else
        {
            $this->addOption(CURLOPT_POSTFIELDS, $data);
        }

        return true;
    }

    protected function addBasicAuthentication($username, $password)
    {
        if ($username === null || $password === null) {
            return false;
        }

        $this->addOption(CURLOPT_USERPWD, $username . ":" . $password);
        return true;
    }

    protected function addBearerAuthentication($token)
    {
        if ($token === null) {
            return false;
        }

        $this->addHeader("Authorization: Bearer " . $token);

        return true;
    }

    protected function send()
    {
        // init curl
        $this->initCurl();

        // prepare options
        $this->prepareOptions();

        // prepare header
        $this->prepareHeader();

        // exec curl
        $result = curl_exec($this->curl);

        // set last request info
        $this->prepareLastRequest();

        $test = curl_error($this->curl);

        // close curl
        $this->closeCurl();

        return $result;
    }

    protected function get($url)
    {
        if (empty($url)) {
            return false;
        }

        // set options
        $this->addOption(CURLOPT_URL, $url);

        // send request
        $result = $this->send();

        return $result;
    }

    protected function post($url, $data, $data_type)
    {
        if (empty($url)) {
            return false;
        }

        // set options
        $this->addOption(CURLOPT_URL, $url);
        $this->addOption(CURLOPT_POST, true);

        // add data
        $this->addData($data, $data_type);

        // send request
        $result = $this->send();

        return $result;
    }

    protected function put($url, $data, $data_type)
    {
        if (empty($url)) {
            return false;
        }

        // set options
        $this->addOption(CURLOPT_URL, $url);
        $this->addOption(CURLOPT_PUT, 1);

        // add data
        $this->addData($data, $data_type);

        // send request
        $result = $this->send();

        return $result;
    }

    protected function delete($url)
    {
        if (empty($url)) {
            return false;
        }

        // set options
        $this->addOption(CURLOPT_URL, $url);
        $this->addOption(CURLOPT_CUSTOMREQUEST, "DELETE");

        // send request
        $result = $this->send();

        return $result;
    }
}
