<?php

namespace DeutschePost\Internetmarke\Admin;

use Exception;
use DeutschePost\Internetmarke\Address;
use DeutschePost\Internetmarke\Enums\ShipmentAttachment;
use DeutschePost\Internetmarke\Integrations\Host\OneClickForApp as OneClickForAppIntegration;
use DeutschePost\Internetmarke\Integrations\Host\OneClickForRefund as OneClickForRefundIntegration;
use DeutschePost\Internetmarke\Models\Resources\Shipment as ShipmentResourceModel;
use DeutschePost\Internetmarke\Products\AdditionalProductList;
use DeutschePost\Internetmarke\Products\SalesProductList;
use DeutschePost\Internetmarke\Repositories\Credentials\User as UserCredentialsRepository;
use DeutschePost\Internetmarke\Repositories\Resources\Shipment;
use DeutschePost\Internetmarke\Repositories\Resources\Shipment as ShipmentResourcesRepository;
use DeutschePost\Internetmarke\Store;
use WC_Order;
use WP_Post;
use function DeutschePost\Internetmarke\Internetmarke;

defined('ABSPATH') || exit;

class MetaBoxes
{
    /**
     * @var OneClickForAppIntegration
     */
    protected $oneClickForApp;

    /**
     * @var OneClickForRefundIntegration
     */
    protected $oneClickForRefund;

    /**
     * @var Store
     */
    protected $store;

    /**
     * MetaBoxes constructor.
     *
     * @param OneClickForAppIntegration    $oneClickForApp    Host integration for OneClickForApp
     * @param OneClickForRefundIntegration $oneClickForRefund Host integration for OneClickForRefund
     * @param Store                        $store             Store information class
     */
    public function __construct(
        OneClickForAppIntegration $oneClickForApp,
        OneClickForRefundIntegration $oneClickForRefund,
        Store $store
    ) {
        $this->oneClickForApp = $oneClickForApp;
        $this->oneClickForRefund = $oneClickForRefund;
        $this->store = $store;

        add_action('add_meta_boxes_shop_order',              array($this, 'addMetaBoxes'), 10, 2);
        add_action('admin_footer-post.php',                  array($this, 'embedShipmentForm'));

        add_action('wp_ajax_wcdpi_prepare_shipment_form',     array($this, 'prepareShipmentForm'));
        add_action('wp_ajax_wcdpi_prepare_shipment_meta_box', array($this, 'prepareShipmentMetaBox'));
        add_action('wp_ajax_wcdpi_quick_buy_change_label',    array($this, 'quickBuyLabelChange'));

        add_action('admin_post_wcdpi_submit_shipment_form',   array($this, 'submitShipmentForm'));

        add_action('save_post_shop_order', array($this, 'saveShopOrder' ), 10);
    }

    /**
     * @param WP_Post $post
     */
    public function addMetaBoxes($post)
    {
        if (!UserCredentialsRepository::instance()->check()
            || !wcdpi_is_woocommerce_installed()
            || !current_user_can('manage_internetmarke')
        ) {
            return;
        }

        $orderId = intval($post->ID);
        $order   = wc_get_order($orderId);

        if (!$order || $order->get_status() == 'auto-draft') {
            return;
        }

        add_meta_box(
            'wcdpi-order-shipment-meta-box',
            'Internetmarke - ' . __('Shipments', 'woo-dp-internetmarke'),
            array($this, 'orderShipmentMetaBox'),
            'shop_order',
            'normal',
            null,
            array('__block_editor_compatible_meta_box' => false)
        );

        add_meta_box(
            'wcdpi-order-quick-buy-meta-box',
            'Internetmarke - ' . __('Quick-buy', 'woo-dp-internetmarke'),
            array($this, 'quickBuyMetaBox'),
            'shop_order',
            'side',
            null,
            array('__block_editor_compatible_meta_box' => false)
        );
    }

    /**
     * Prepares and includes the shipment meta box template file.
     *
     * @param WP_Post $post
     */
    public function orderShipmentMetaBox($post)
    {
        /** @var WC_Order $order */
        $order = wc_get_order($post->ID);

        if (!$order) {
            return;
        }

        $orderShipments = ShipmentResourcesRepository::instance()->findAllBy(
            array(
                'order_id' => $order->get_id(),
            ),
            'id',
            'DESC'
        );

        include_once WCDPI_TEMPLATES_DIR . 'meta-box-order-shipments.php';
    }

    /**
     * Prepares and includes the quick-buy meta box template file.
     *
     * @param WP_Post $post
     */
    public function quickBuyMetaBox($post)
    {
        /** @var WC_Order $order */
        $order = wc_get_order($post->ID);

        if (!$order) {
            return;
        }

        $shippingCountry = $order->get_shipping_country();

        include_once WCDPI_TEMPLATES_DIR . 'meta-box-quick-buy.php';
    }

    /**
     * Callback for the AJAX action "prepare_shipment_form".
     */
    public function prepareShipmentForm()
    {
        $orderId      = intval($_POST['orderId']);
        $orderAddress = $this->store->getOrderAddress($orderId);
        $storeAddress = $this->store->getStoreAddress();
        $menuList     = wcdpi_get_product_list_navigation();
        $productList  = new SalesProductList();
        $optionsList  = new AdditionalProductList();
        $lineItems    = Internetmarke()->shipmentItemService->prepare($orderId);

        $data = array(
            'orderId'           => $orderId,
            'sender'            => $storeAddress->toArray(),
            'recipient'         => $orderAddress->toArray(),
            'currencySymbol'    => get_woocommerce_currency_symbol(),
            'decimalSeparator'  => wc_get_price_decimal_separator(),
            'thousandSeparator' => wc_get_price_thousand_separator(),
            'decimals'          => wc_get_price_decimals(),
            'priceFormat'       => get_woocommerce_price_format(),
            'menuList'          => $menuList,
            'productList'       => $productList,
            'optionsList'       => $optionsList->getAll(),
            'lineItems'         => $lineItems,
            'weightUnit'        => 'g',
        );

        wp_send_json($data);
    }

    /**
     * Callback for the AJAX action "prepare_shipment_meta_box".
     */
    public function prepareShipmentMetaBox()
    {
        global $wpdb;

        $orderId = intval($_POST['orderId']);
        $labelAttachmentType = ShipmentAttachment::LABEL;

        $sqlQuery = "
            SELECT
                shipment.`id` AS id
                , shipment.`status` AS status
                , shipment.`tracking` AS tracking
                , (
                    SELECT
                        SUM(shipment_item.`product_weight` * shipment_item.`product_quantity`)
                    FROM
                        {$wpdb->prefix}wcdpi_shipment_item AS shipment_item
                    WHERE
                        shipment.`id` = shipment_item.`shipment_id`
                  ) AS weight
                , (
                    SELECT
                        SUM(shipment_item.`product_price` * shipment_item.`product_quantity`)
                    FROM
                        {$wpdb->prefix}wcdpi_shipment_item AS shipment_item
                    WHERE
                        shipment.`id` = shipment_item.`shipment_id`
                ) AS value
                , (
                    SELECT
                        shipment_attachment.`url`
                    FROM
                        {$wpdb->prefix}wcdpi_shipment_attachment AS shipment_attachment
                    WHERE
                        1=1
                        AND shipment.`id` = shipment_attachment.shipment_id
                        AND shipment_attachment.`type` = {$labelAttachmentType}
                ) AS shippingLabelUrl
                , shipment.`meta_insert_dt` AS createdAt
                , shipment.`meta_update_dt` AS updatedAt
            FROM
                {$wpdb->prefix}wcdpi_shipment AS shipment
            WHERE
                1=1
                AND shipment.order_id = {$orderId}
            ;
        ";

        /** @var ShipmentResourceModel[] $shipments */
        $shipments = ShipmentResourcesRepository::instance()->query($sqlQuery, ARRAY_A);

        $data = array(
            'shipments'         => $shipments,
            'currencySymbol'    => get_woocommerce_currency_symbol(),
            'decimalSeparator'  => wc_get_price_decimal_separator(),
            'thousandSeparator' => wc_get_price_thousand_separator(),
            'decimals'          => wc_get_price_decimals(),
            'priceFormat'       => get_woocommerce_price_format()
        );

        wp_send_json($data);
    }

    /**
     * Callback for the AJAX action "quick_buy_label_change".
     */
    public function quickBuyLabelChange()
    {
        $order = wc_get_order($_POST['orderId']);
        $prodwsId = $_POST['prodwsId'];

        if (!$order || !$order instanceof WC_Order) {
            wp_send_json(
                array(
                    'isHarmonizedLabelAvailable' => false
                )
            );

            wp_die();
        }

        $isHarmonizedLabelAvailable = Internetmarke()->labelService->isHarmonizedLabelAvailable($order, $prodwsId);

        wp_send_json(
            array(
                'isHarmonizedLabelAvailable' => $isHarmonizedLabelAvailable
            )
        );

        wp_die();
    }

    /**
     * Prepares and includes the wizard template file.
     */
    public function embedShipmentForm()
    {
        global $post;

        if ($post->post_type != 'shop_order') {
            return;
        }

        $orderId = intval($post->ID);
        $wooCountries = new \WC_Countries();
        $countries = $wooCountries->get_countries();

        include_once WCDPI_TEMPLATES_DIR . 'modal-shipment-wizard.php';
    }

    /**
     * Forwards the request to the shipment service.
     */
    public function submitShipmentForm()
    {
        $harmonizedLabel = isset($_POST['harmonized_label']);

        $shipment = Internetmarke()->shipmentService->prepare(
            $_POST['orderId'],
            $_POST['prodwsId'],
            $harmonizedLabel,
            new Address($_POST['sender']),
            new Address($_POST['recipient']),
            $_POST['lineItems']
        );

        if (!$shipment) {
            Notice::error(__('Your shipment was not prepared.', 'woo-dp-internetmarke'));
            wp_redirect(admin_url(sprintf('post.php?post=%d&action=edit', intval($_POST['orderId']))));
            exit();
        }

        if ($_POST['shipmentAction'] === 'prepare') {
            Notice::success(__('Your shipment was prepared successfully.', 'woo-dp-internetmarke'));
            wp_redirect(admin_url(sprintf('post.php?post=%d&action=edit', intval($_POST['orderId']))));
            exit();
        }

        try {
            Internetmarke()->shipmentService->order($shipment->getId());
        } catch (Exception $e) {
            wcdpi_set_shipment_status_failed($shipment->getId());
            wcdpi_add_shipment_failed_order_note($shipment, $e);
            Internetmarke()->logger()->error($e->getMessage());
            Notice::error(
                sprintf(
                    '%s %s: %s',
                    __('Your shipment was not created.', 'woo-dp-internetmarke'),
                    __('Error', 'woo-dp-internetmarke'),
                    $e->getMessage()
                )
            );
            wp_redirect(admin_url(sprintf('post.php?post=%d&action=edit', intval($_POST['orderId']))));
            exit();
        }

        Notice::success(__('Your shipment was created successfully.', 'woo-dp-internetmarke'));
        wp_redirect(admin_url(sprintf('post.php?post=%d&action=edit', intval($_POST['orderId']))));
        exit();
    }

    /**
     * Processes the "save_post" WordPress action.
     *
     * @param int $postId
     *
     * @throws Exception
     */
    public function saveShopOrder($postId)
    {
        // Quick buy shipment action
        if (array_key_exists('wcdpi_shipment_action_quick_buy', $_POST)) {
            $lineItems       = Internetmarke()->shipmentItemService->prepare($postId);
            $harmonizedLabel = false;
            $orderAddress    = $this->store->getOrderAddress($postId);
            $storeAddress    = $this->store->getStoreAddress();
            $prodWsId        = $_POST['wcdpi_quick_buy_shipping_product'];

            if (!$prodWsId) {
                throw new Exception('No shipping product selected');
            }

            if (array_key_exists('wcdpi_quick_buy_harmonized_label', $_POST)
                && $_POST['wcdpi_quick_buy_harmonized_label']
            ) {
                $harmonizedLabel = true;
            }

            $shipment = Internetmarke()->shipmentService->prepare(
                $postId,
                $prodWsId,
                $harmonizedLabel,
                $storeAddress,
                $orderAddress,
                $lineItems
            );

            if (!$shipment) {
                Notice::error(__('Your shipment was not prepared.', 'woo-dp-internetmarke'));
                wp_redirect(admin_url(sprintf('post.php?post=%d&action=edit', $postId)));
                exit();
            }

            try {
                Internetmarke()->shipmentService->order($shipment->getId());
            } catch (Exception $e) {
                wcdpi_set_shipment_status_failed($shipment->getId());
                wcdpi_add_shipment_failed_order_note($shipment, $e);
                Internetmarke()->logger()->error($e->getMessage());
                Notice::error(
                    sprintf(
                        sprintf(
                            '%s %s: %s',
                            __('Your shipment was not created.', 'woo-dp-internetmarke'),
                            __('Error', 'woo-dp-internetmarke'),
                            $e->getMessage()
                        )
                    )
                );
                wp_redirect(admin_url(sprintf('post.php?post=%d&action=edit', $postId)));
                exit();
            }

            Notice::success(__('Your shipment was created successfully.', 'woo-dp-internetmarke'));
            wp_redirect(admin_url(sprintf('post.php?post=%d&action=edit', $postId)));
            exit();
        }

        // Delete shipments action
        if (array_key_exists('wcdpi_shipment_action_mark_as_deleted', $_POST)) {
            if (!array_key_exists('wcdpi_selected_shipments', $_POST)) {
                Notice::warning(__('You did not select any shipments.', 'woo-dp-internetmarke'));
                wp_redirect(admin_url(sprintf('post.php?post=%d&action=edit', $postId)));
                exit();
            }

            $selectedShipments = (array)$_POST['wcdpi_selected_shipments'];

            $result = Internetmarke()->shipmentService->delete($selectedShipments);

            if (!$result) {
                Notice::error('Your shipments were not deleted.');
                wp_redirect(admin_url(sprintf('post.php?post=%d&action=edit', $postId)));
                exit();
            }

            Notice::success(sprintf(__('%d shipment(s) deleted successfully.', 'woo-dp-internetmarke'), $result));
            wp_redirect(admin_url(sprintf('post.php?post=%d&action=edit', $postId)));
            exit();
        }

        // Order labels action
        if (array_key_exists('wcdpi_shipment_action_order_labels', $_POST)) {
            if (!array_key_exists('wcdpi_selected_shipments', $_POST)) {
                Notice::warning(__('You did not select any shipments.', 'woo-dp-internetmarke'));
                wp_redirect(admin_url(sprintf('post.php?post=%d&action=edit', $postId)));
                exit();
            }

            $selectedShipments = (array)$_POST['wcdpi_selected_shipments'];
            $count = 0;

            foreach ($selectedShipments as $shipmentId) {
                /** @var false|ShipmentResourceModel $shipment */
                $shipment = ShipmentResourcesRepository::instance()->find($shipmentId);

                if (!$shipment) {
                    continue;
                }

                try {
                    Internetmarke()->shipmentService->order($shipmentId);
                } catch (Exception $e) {
                    wcdpi_set_shipment_status_failed($shipmentId);
                    wcdpi_add_shipment_failed_order_note($shipment, $e);
                    Internetmarke()->logger()->error($e->getMessage(), $e->getTrace());
                    continue;
                }

                $count++;
            }

            if ($count === 0) {
                Notice::error('Your shipments were not ordered.');
                wp_redirect(admin_url(sprintf('post.php?post=%d&action=edit', $postId)));
                exit();
            }

            Notice::success(sprintf(__('%d shipment(s) deleted successfully.', 'woo-dp-internetmarke'), $count));
            wp_redirect(admin_url(sprintf('post.php?post=%d&action=edit', $postId)));
            exit();
        }

        // Refund labels action
        if (array_key_exists('wcdpi_shipment_action_refund_label', $_POST)) {
            if (!array_key_exists('wcdpi_selected_shipments', $_POST)) {
                Notice::warning(__('You did not select any shipments.', 'woo-dp-internetmarke'));
                wp_redirect(admin_url(sprintf('post.php?post=%d&action=edit', $postId)));
                exit();
            }

            $selectedShipments = (array)$_POST['wcdpi_selected_shipments'];
            $count = 0;

            foreach ($selectedShipments as $shipmentId) {
                if (Internetmarke()->shipmentService->cancel($shipmentId)) {
                    $count++;
                }
            }

            if ($count === 0) {
                Notice::error('Your shipments were not cancelled.');
                wp_redirect(admin_url(sprintf('post.php?post=%d&action=edit', $postId)));
                exit();
            }

            Notice::success(sprintf(__('%d shipment(s) cancelled successfully.', 'woo-dp-internetmarke'), $count));
            wp_redirect(admin_url(sprintf('post.php?post=%d&action=edit', $postId)));
            exit();
        }

        // Mark as shipped
        if (array_key_exists('wcdpi_shipment_action_mark_as_shipped', $_POST)) {
            if (!array_key_exists('wcdpi_selected_shipments', $_POST)) {
                Notice::warning(__('You did not select any shipments.', 'woo-dp-internetmarke'));
                wp_redirect(admin_url(sprintf('post.php?post=%d&action=edit', $postId)));
                exit();
            }

            $selectedShipments = (array)$_POST['wcdpi_selected_shipments'];
            $count = 0;

            foreach ($selectedShipments as $shipmentId) {
                /** @var ShipmentResourceModel $shipment */
                $shipment = ShipmentResourcesRepository::instance()->find($shipmentId);

                if ($shipment && $shipment->getOrderId()) {
                    $order = wc_get_order($shipment->getOrderId());

                    if ($order) {
                        $order->update_status('completed');
                        $order->save();
                    }
                }

                if (wcdpi_set_shipment_status_shipped($shipmentId)) {
                    $count++;
                }
            }

            Notice::success(sprintf(__('%d shipment(s) marked as shipped.', 'woo-dp-internetmarke'), $count));
            wp_redirect(admin_url(sprintf('post.php?post=%d&action=edit', $postId)));
            exit();
        }

        // Mark as delivered
        if (array_key_exists('wcdpi_shipment_action_mark_as_delivered', $_POST)) {
            if (!array_key_exists('wcdpi_selected_shipments', $_POST)) {
                Notice::warning(__('You did not select any shipments.', 'woo-dp-internetmarke'));
                wp_redirect(admin_url(sprintf('post.php?post=%d&action=edit', $postId)));
                exit();
            }

            $selectedShipments = (array)$_POST['wcdpi_selected_shipments'];
            $count = 0;

            foreach ($selectedShipments as $shipmentId) {
                if (wcdpi_set_shipment_status_deleted($shipmentId)) {
                    $count++;
                }
            }

            Notice::success(sprintf(__('%d shipment(s) marked as delivered.', 'woo-dp-internetmarke'), $count));
            wp_redirect(admin_url(sprintf('post.php?post=%d&action=edit', $postId)));
            exit();
        }

        // Mark as returned
        if (array_key_exists('wcdpi_shipment_action_mark_as_returned', $_POST)) {
            if (!array_key_exists('wcdpi_selected_shipments', $_POST)) {
                Notice::warning(__('You did not select any shipments.', 'woo-dp-internetmarke'));
                wp_redirect(admin_url(sprintf('post.php?post=%d&action=edit', $postId)));
                exit();
            }

            $selectedShipments = (array)$_POST['wcdpi_selected_shipments'];
            $count = 0;

            foreach ($selectedShipments as $shipmentId) {
                if (wcdpi_set_shipment_status_returned($shipmentId)) {
                    $count++;
                }
            }

            Notice::success(sprintf(__('%d shipment(s) marked as returned.', 'woo-dp-internetmarke'), $count));
            wp_redirect(admin_url(sprintf('post.php?post=%d&action=edit', $postId)));
            exit();
        }
    }
}