<?php


namespace DeutschePost\Internetmarke\Components;


use DeutschePost\Internetmarke\Address;
use DeutschePost\Internetmarke\Admin\Notice;
use DeutschePost\Internetmarke\Interfaces\ComponentIntegration as ComponentIntegrationInterface;
use DeutschePost\Internetmarke\Products\AdditionalProductList;
use DeutschePost\Internetmarke\Products\SalesProductList;
use DeutschePost\Internetmarke\Repositories\Credentials\User as UserCredentialsRepository;
use DeutschePost\Internetmarke\Store;
use Exception;
use WC_Countries;
use WP_User;
use WP_User_Query;
use function DeutschePost\Internetmarke\Internetmarke;

defined('ABSPATH') || exit;

class LabelShop implements ComponentIntegrationInterface
{
    /**
     * @var Store
     */
    protected $store;

    /**
     * LabelShop constructor.
     *
     * @param Store $store
     */
    public function __construct(Store $store)
    {
        $this->store = $store;

        add_action('admin_menu', array($this, 'init'), 10);

        add_action('admin_post_wcdpi_label_shop_submit_shipment',    array($this, 'submitShipment'),      10, 1);
        add_action('wp_ajax_wcdpi_label_shop_prepare_form',          array($this, 'prepareForm'),         10, 1);
        add_action('wp_ajax_wcdpi_label_shop_load_default_sender',   array($this, 'loadDefaultSender'),   10, 1);
        add_action('wp_ajax_wcdpi_label_shop_load_customer_address', array($this, 'loadCustomerAddress'), 10, 1);
        add_action('wp_ajax_wcdpi_label_shop_add_line_item',         array($this, 'addLineItem'),         10, 1);
        add_action('wp_ajax_wcdpi_dropdown_user_search',             array($this, 'dropdownUserSearch'),  10, 1);
    }

    public function init()
    {
        if (!UserCredentialsRepository::instance()->check()) {
            return;
        }

        add_submenu_page(
            'internetmarke',
            __('Label shop', 'woo-dp-internetmarke'),
            __('Label shop', 'woo-dp-internetmarke'),
            'manage_internetmarke',
            'internetmarke_label_shop',
            array($this, 'showPage')
        );
    }

    public function showPage()
    {
        $wooCountries = new WC_Countries();
        $countries    = $wooCountries->get_countries();

        include_once WCDPI_TEMPLATES_DIR . 'page-label-shop.php';
    }

    public function submitShipment()
    {
        $harmonizedLabel = isset($_POST['harmonized_label']);
        $lineItems = array();

        if (isset($_POST['lineItems'])) {
            $lineItems = $_POST['lineItems'];
        }

        $shipment = Internetmarke()->shipmentService->prepare(
            $_POST['orderId'],
            $_POST['prodwsId'],
            $harmonizedLabel,
            new Address($_POST['sender']),
            new Address($_POST['recipient']),
            $lineItems
        );

        if (!$shipment) {
            Notice::error(__('Your shipment was not prepared.', 'woo-dp-internetmarke'));
            wp_redirect(admin_url('admin.php?page=internetmarke_label_shop'));
            exit();
        }

        if ($_POST['shipmentAction'] === 'prepare') {
            Notice::success(__('Your shipment was prepared successfully.', 'woo-dp-internetmarke'));
            wp_redirect(admin_url('admin.php?page=internetmarke_label_shop'));
            exit();
        }

        try {
            Internetmarke()->shipmentService->order($shipment->getId());
        } catch (Exception $e) {
            wcdpi_set_shipment_status_failed($shipment->getId());
            wcdpi_add_shipment_failed_order_note($shipment, $e);
            Internetmarke()->logger()->error($e->getMessage());
            Notice::error(
                sprintf(
                    '%s %s: %s',
                    __('Your shipment was not created.', 'woo-dp-internetmarke'),
                    __('Error', 'woo-dp-internetmarke'),
                    $e->getMessage()
                )
            );
            wp_redirect(admin_url('admin.php?page=internetmarke_label_shop'));
            exit();
        }

        Notice::success(__('Your shipment was created successfully.', 'woo-dp-internetmarke'));
        wp_redirect(admin_url('admin.php?page=internetmarke_label_shop'));
        exit();
    }

    public function prepareForm()
    {
        $menuList     = wcdpi_get_product_list_navigation();
        $productList  = new SalesProductList();
        $optionsList  = new AdditionalProductList();

        $data = array(
            'currencySymbol'    => get_woocommerce_currency_symbol(),
            'decimalSeparator'  => wc_get_price_decimal_separator(),
            'thousandSeparator' => wc_get_price_thousand_separator(),
            'decimals'          => wc_get_price_decimals(),
            'priceFormat'       => get_woocommerce_price_format(),
            'menuList'          => $menuList,
            'productList'       => $productList,
            'optionsList'       => $optionsList->getAll(),
            'lineItems'         => array(),
            'weightUnit'        => 'g',
            'euCountries'       => wcdpi_get_eu_countries()
        );

        wp_send_json($data);
    }

    public function loadDefaultSender()
    {
        wp_send_json($this->store->getStoreAddress()->toArray());
    }

    public function loadCustomerAddress()
    {
        $userId = (int)$_POST['userId'];
        $addressType = sanitize_text_field($_POST['addressType']);

        wp_send_json($this->store->getUserAddress($userId, $addressType)->toArray());
    }

    public function addLineItem()
    {
        check_ajax_referer('wcdpi_label_shop_add_line_item', 'security');

        $productId = (int)$_POST['productId'];

        if (!$product = wc_get_product($productId)) {
            wp_send_json_error(array('message' => __('Product not found.', 'woo-dp-internetmarke')));
        }

        $customsDeclarationText = get_post_meta($productId, 'wcdpi_customs_declaration_text', true);

        if (!$customsDeclarationText) {
            $customsDeclarationText = substr($product->get_name(), 0, 33);
        }

        wp_send_json(
            array(
                'productId'            => $productId,
                'lineItemId'           => $productId,
                'orderId'              => null,
                'editUrl'              => get_edit_post_link($productId, '&'),
                'name'                 => $product->get_name(),
                'sku'                  => $product->get_sku(),
                'weight'               => wcdpi_convert_weight(get_option('woocommerce_weight_unit'), 'g', $product->get_weight()),
                'price'                => $product->get_price(),
                'dimensions'           => wc_format_dimensions($product->get_dimensions(false)),
                'qtySelected'          => 1,
                'countryOfManufacture' => get_post_meta($productId, 'wcdpi_customs_declaration_origin', true),
                'hsCode'               => get_post_meta($productId, 'wcdpi_customs_declaration_hs_code', true),
                'customsDeclaration'   => $customsDeclarationText,
            )
        );
    }

    public function dropdownUserSearch()
    {
        check_ajax_referer('wcdpi_dropdown_user_search', 'security');

        $page    = 1;
        $limit   = 10;
        $results = array();
        $term    = isset($_POST['term']) ? (string)wc_clean(wp_unslash($_POST['term'])) : '';
        $more    = true;
        $offset  = 0;

        if (!$term) {
            wp_die();
        }

        if (array_key_exists('page', $_POST)) {
            $page = (int)$_POST['page'];
        }

        if ($page > 1) {
            $offset = ($page - 1) * $limit;
        }

        $users = new WP_User_Query(
            array(
                'search'         => '*'.esc_attr($term).'*',
                'search_columns' => array(
                    'user_id',
                    'user_login',
                    'user_nicename',
                    'user_email',
                    'user_url',
                ),
                'number' => $limit,
                'offset' => $offset
            )
        );

        if (count($users->get_results()) < $limit) {
            $more = false;
        }

        /** @var WP_User $user */
        foreach ($users->get_results() as $user) {
            $results[] = array(
                'id' => $user->ID,
                'text' => sprintf(
                /* translators: $1: user nicename, $2: user id, $3: user email */
                    '%1$s (#%2$d - %3$s)',
                    $user->user_nicename,
                    $user->ID,
                    $user->user_email
                )
            );
        }

        wp_send_json(
            array(
                'page'       => $page,
                'results'    => $results,
                'pagination' => array(
                    'more' => $more
                )
            )
        );
    }
}