<?php

namespace DeutschePost\Internetmarke\Components;

use DeutschePost\Internetmarke\Admin\Notice;
use DeutschePost\Internetmarke\Connectors\PacketPlus\Response\GetItemLabel as GetItemLabelResponse;
use DeutschePost\Internetmarke\Enums\ShippingRegion as ShippingRegionEnum;
use DeutschePost\Internetmarke\Integrations\Host\PacketPlus as PacketPlusHostIntegration;
use DeutschePost\Internetmarke\Interfaces\ComponentIntegration as ComponentIntegrationInterface;
use DeutschePost\Internetmarke\Repositories\Credentials\User as UserCredentialsRepository;
use DeutschePost\Internetmarke\Store;
use Exception;
use WC_Countries;
use WC_Order;
use function DeutschePost\Internetmarke\Internetmarke;

defined('ABSPATH') || exit;

class Shipments implements ComponentIntegrationInterface
{
    /**
     * @var Store
     */
    protected $store;

    /**
     * @var PacketPlusHostIntegration
     */
    private $packetPlusHostIntegration;

    /**
     * Shipments constructor.
     *
     * @param Store                     $store
     * @param PacketPlusHostIntegration $packetPlusHostIntegration
     */
    function __construct(
        Store $store,
        PacketPlusHostIntegration $packetPlusHostIntegration
    ) {
        $this->store = $store;
        $this->packetPlusHostIntegration = $packetPlusHostIntegration;

        add_action('admin_menu',                                  array($this, 'init'),            10);
        add_filter('set-screen-option',                           array($this, 'setScreenOption'), 10, 3);
        add_action('admin_post_wcdpi_download_packet_plus_label', array($this, 'downloadPacketPlusLabel'));

        add_action('wp_ajax_wcdpi_init_single_mode', array($this, 'ajaxInitSingleLabelMode'));
        add_action('wp_ajax_wcdpi_init_multi_mode',  array($this, 'ajaxInitMultiLabelMode'));
        add_action('wp_ajax_wcdpi_shipment_submit',  array($this, 'ajaxShipmentSubmit'));
    }

    /**
     * Initialize the page and corresponding hooks.
     */
    public function init()
    {
        if (!UserCredentialsRepository::instance()->check()) {
            return;
        }

        $shipmentsViewHook = add_submenu_page(
            'internetmarke',
            __('Shipments', 'woo-dp-internetmarke'),
            __('Shipments', 'woo-dp-internetmarke'),
            'manage_internetmarke',
            'internetmarke_shipments',
            array($this, 'showPage')
        );

        $shipmentsBulkViewHook = add_submenu_page(
            null,
            __('Shipments - Bulk processing', 'woo-dp-internetmarke'),
            __('Shipments - Bulk processing', 'woo-dp-internetmarke'),
            'manage_internetmarke',
            'internetmarke_shipments_bulk_create',
            array($this, 'showBulkPage')
        );

        add_action('load-' . $shipmentsViewHook, array($this, 'pageScreenOptions'));
    }

    /**
     * Modifies the "per_page" setting.
     *
     * @see https://developer.wordpress.org/reference/hooks/set-screen-option/
     *
     * @param bool   $keep   Whether to save or skip saving the screen option value (default false)
     * @param string $option The option name.
     * @param int    $value  The number of rows to use
     *
     * @return int
     */
    public function setScreenOption($keep, $option, $value)
    {
        return $value;
    }

    /**
     * Displays the shipments table.
     */
    public function showPage()
    {
        $shipments = new View\Shipments(
            $this->store,
            $this->packetPlusHostIntegration
        );

        include_once WCDPI_TEMPLATES_DIR . 'page-shipments.php';
    }

    public function showBulkPage()
    {
        $cookie       = $_COOKIE['internetmarke_shipments_bulk_create'];
        $orderData    = json_decode(stripslashes($cookie));
        $destination  = ShippingRegionEnum::DOMESTIC;
        $region       = strtoupper($orderData->destination);
        $orders       = $orderData->orders;
        $recipients   = $this->getOrderData($orders, $destination);
        $wooCountries = new WC_Countries();
        $countries    = $wooCountries->get_countries();
        $ordersValue  = htmlspecialchars(json_encode($orders, JSON_HEX_APOS), ENT_QUOTES, 'UTF-8');

        // No region was selected, display all labels
        if (!$region) {
            $destination = null;
        }

        // Any EU country
        elseif ($region === ShippingRegionEnum::EUROPEAN_UNION) {
            $destination = 'AT';
        }

        // Any other country
        elseif ($region === ShippingRegionEnum::REST_OF_WORLD) {
            $destination = 'US';
        }

        include_once WCDPI_TEMPLATES_DIR . 'page-shipments-bulk-create.php';
    }

    /**
     * Displays the screen options.
     */
    public function pageScreenOptions()
    {
        $option = 'per_page';

        $args = array(
            'label'   => __('Shipments per page', 'woo-dp-internetmarke'),
            'default' => 10,
            'option'  => 'wcdpi_shipments_per_page'
        );

        add_screen_option($option, $args);
    }

    /**
     * Performs a harmonized label download.
     */
    public function downloadPacketPlusLabel()
    {
        // get label parameter
        $labelId = array_key_exists('item', (array)$_GET) ? intval($_GET['item']) : '';

        if (!$labelId) {
            Notice::error(__('Error during label download.', 'woo-dp-internetmarke'));
            wp_redirect(admin_url('admin.php?page=internetmarke'));
            exit();
        }

        /** @var GetItemLabelResponse $label */
        $label = $this->packetPlusHostIntegration->getLabel($labelId);
        $fileName = 'HarmonizedLabel-' . $labelId . '.pdf';

        ob_clean();
        status_header(200);
        header("Content-Type: application/pdf");
        header("Content-Disposition:attachment;filename=" . $fileName);
        echo $label->getLabel();
        exit();
    }

    /**
     * Prepares the order data for display in the frontend.
     *
     * @param int[]  $orderIds    Array of WooCommerce order ID's
     * @param string $destination ShippingStatus Enum values
     *
     * @return array
     */
    private function getOrderData($orderIds, $destination)
    {
        $result = array();

        foreach ($orderIds as $orderId) {
            /** @var WC_Order $order */
            $order = wc_get_order($orderId);

            $shippingCountry = $order->get_shipping_country();

            $result[] = array(
                'orderId'     => $orderId,
                'destination' => $shippingCountry,
                'total'       => $order->get_formatted_order_total()
            );
        }

        return $result;
    }

    public function ajaxInitSingleLabelMode()
    {
        $result = array();
        $orders = $_POST['orders'];

        foreach ($orders as $orderId) {
            $order = wc_get_order($orderId);

            if (!$order) {
                continue;
            }

            $result[] = array(
                'id'      => $order->get_id(),
                'editUrl' => $order->get_edit_order_url(),
                'number'  => $order->get_order_number(),
                'email'   => $order->get_billing_email(),
                'address' => $order->get_formatted_shipping_address(),
                'active'  => false
            );
        }

        wp_send_json($result);
    }

    public function ajaxShipmentSubmit()
    {
        $orderedShipment = false;
        $action          = sanitize_text_field($_POST['type']);
        $harmonizedLabel = false;
        $orderId         = absint($_POST['orderId']);
        $prodwsId        = sanitize_text_field($_POST['prodwsId']);
        $lineItems       = Internetmarke()->shipmentItemService->prepare($orderId);
        $recipient       = $this->store->getOrderAddress($orderId);
        $sender          = $this->store->getStoreAddress();

        if (array_key_exists('harmonizedLabel', $_POST)) {
            $harmonizedLabel = filter_var($_POST['harmonizedLabel'], FILTER_VALIDATE_BOOLEAN);
        }

        $preparedShipment = Internetmarke()->shipmentService->prepare(
            $orderId,
            $prodwsId,
            $harmonizedLabel,
            $sender,
            $recipient,
            $lineItems
        );

        if ($action === 'create') {
            try {
                $orderedShipment = Internetmarke()->shipmentService->order($preparedShipment->getId());
            } catch (Exception $e) {
                wcdpi_set_shipment_status_failed($preparedShipment->getId());
                wcdpi_add_shipment_failed_order_note($preparedShipment, $e);
                Internetmarke()->logger()->error($e->getMessage());
                wp_send_json($e->getMessage(), 500);
                return;
            }
        }

        if ($orderedShipment !== false) {
            $shipment = $orderedShipment->toArray();
            $shipment['shippingLabelUrl'] = $orderedShipment->getLabel();
            wp_send_json($shipment);
        } else {
            wp_send_json($preparedShipment->toArray());
        }
    }
}