<?php

namespace DeutschePost\Internetmarke\Connectors\REST;

use Exception;
use DeutschePost\Internetmarke\Abstracts\RestCurl;
use DeutschePost\Internetmarke\Enums\HttpDataType;
use WP_Error;
use function DeutschePost\Internetmarke\Internetmarke;

defined('ABSPATH') || exit;

class FORKLane extends RestCurl
{
    /**
     * Concatenated authentication GET parameters
     *
     * @var string
     */
    private $auth;

    /**
     * Legacy remote service URL from which the credentials are obtained.
     *
     * @var string
     */
    const REMOTE_SERVICE_URL = 'https://api.fork-lane.com/api/p00011/domain?Key=7cb5d6be-58f6-45c0-beb4-7b5aab854d47&Id=b91c7b40-943b-4437-899a-74227df381e6';

    /**
     * Portfolio Element ID
     * 
     * @var string
     */
    const PORTFOLIO_ELEMENT = 'p00027';

    /**
     * Marketplace number
     * 
     * @var string
     */
    const MARKETPLACE = '1';

    /**
     * Service Remote Request URL
     * 
     * @var string
     */
    const SERVICE_REQUEST_URL = 'https://api.fork-lane.com/v1/p00011';

    /**
     * Service Authentication Key
     * 
     * @var string
     */
    const SERVICE_KEY = '7cb5d6be-58f6-45c0-beb4-7b5aab854d47';

    /**
     * Service Authentication ID
     * 
     * @var string
     */
    const SERVICE_ID = 'b91c7b40-943b-4437-899a-74227df381e6';

    /**
     * FORKLane constructor.
     *
     * @throws Exception
     */
    public function __construct()
    {
        parent::__construct();

        $this->auth = '?Key=' . self::SERVICE_KEY . '&Id=' . self::SERVICE_ID;

        add_filter('wcdpi_get_partner_credentials', array($this, 'getPartnerCredentials'), 10, 0);

        add_filter('wcdpi_internetmarke_activate_plugin', array($this, 'activatePlugin'));
        add_filter('wcdpi_internetmarke_deactivate_plugin', array($this, 'deactivatePlugin'));
        add_filter('wcdpi_internetmarke_update_product_list', array($this, 'updateProductList'));
    }

    /**
     * Clean up variables and prepare for a new request.
     */
    protected function cleanup()
    {
        $this->clearHeader();
        $this->initDefaultHeader();
        $this->clearOptions();
        $this->initDefaultOptions();
    }

    /**
     * Prepares an info object used to communicate with the Fork Lane service.
     * 
     * @param string $event Name of the event
     * 
     * @return array
     */
    private function createInfoObject($event)
    {
        return array(
            'header' => array(
                'id' => '',
                'parent_id' => '',
                'portfolio_element' => self::PORTFOLIO_ELEMENT,
                'client' => '',
                'node' => '',
                'version' => '1-0-1',
                'type' => 'SINGLE',
                'status' => 'NEW',
                'encryption_level' => '0',
                'encryption_key' => '',
                'process_history' => array(
                    array(
                        'step' => '1',
                        'timestamp' => gmdate('Y-m-d H:i:s'),
                        'unit' => '',
                        'type' => 'SERVICE_REQUEST',
                        'status' => 'NEW',
                        'status_comment' => ''
                    )
                )
            ),
            'container' => array(
                array(
                    'header' => array(
                        'id' => '',
                        'parent_id' => '',
                        'portfolio_element' => self::PORTFOLIO_ELEMENT,
                        'client' => '',
                        'node' => '',
                        'version' => '1-0-1',
                        'type' => 'SERVICE_REQUEST',
                        'status' => 'NEW',
                        'encryption_level' => '0',
                        'encryption_key' => ''
                    ),
                    'payload' => array(
                        'siteUrl' => get_site_url(),
                        'marketplace' => self::MARKETPLACE,
                        'event' => $event
                    )
                )
            )
        );
    }

    /**
     * Activates the plugin.
     *
     * @return string
     */
    public function activatePlugin()
    {
        // prepare and send request
        $url = self::SERVICE_REQUEST_URL . '/info' . $this->auth;
        $this->addHeader('Content-Type: application/json');

        $data = $this->createInfoObject('ACTIVATE_PLUGIN');

        $result = $this->post($url, $data, HttpDataType::JSON);

        // cleanup
        $this->cleanup();

        // return data
        return $result;
    }

    /**
     * Deactivates the plugin.
     *
     * @return string
     */
    public function deactivatePlugin()
    {
        // prepare and send request
        $url = self::SERVICE_REQUEST_URL . '/info' . $this->auth;
        $this->addHeader('Content-Type: application/json');

        $data = $this->createInfoObject('DEACTIVATE_PLUGIN');

        $result = $this->post($url, $data, HttpDataType::JSON);

        // cleanup
        $this->cleanup();

        // return data
        return $result;
    }

    /**
     * Updates the product list.
     *
     * @return string
     */
    public function updateProductList()
    {
        // prepare and send request
        $url = self::SERVICE_REQUEST_URL . '/info' . $this->auth;
        $this->addHeader('Content-Type: application/json');

        $data = $this->createInfoObject('UPDATE_PRODUCT_LIST');

        $result = $this->post($url, $data, HttpDataType::JSON);

        // cleanup
        $this->cleanup();

        // return data
        return $result;
    }

    /**
     * Legacy function used to obtain partner credentials from the remote service.
     * 
     * @throws Exception
     * @return string
     */
    public function getPartnerCredentials()
    {
        $request = wp_remote_post(
            self::REMOTE_SERVICE_URL,
            array(
                'method' => 'POST',
                'body'   => array(
                    'site_url'    => get_site_url(),
                    'tac'         => 'true',
                    'marketplace' => 1,
                    'client'      => 1
                ),
                'sslverify' => false
            )
        );

        if ($request instanceof WP_Error) {
            foreach ($request->get_error_messages() as $error_message) {
                $e = new Exception($error_message);
                Internetmarke()->logger()->error($e->getMessage(), $e->getTrace());
            }

            throw new Exception(
                sprintf(__('There has been an error connecting to the remote service, please enable the logger and forward the generated log entries when contacting our support team (<a href="mailto:support@stina-global.com">support@stina-global.com</a> or the <a href="%s" target="_blank">WordPress Support Forum</a>).', 'woo-dp-internetmarke'), 'https://wordpress.org/support/plugin/woo-dp-internetmarke'));
        }

        $response = json_decode($request['body']);

        if ($response->status == 'success') {
            return $response;
        } elseif ($response->status == 'error') {
            throw new Exception('Server returned an error.');
        }
    }

}
