<?php

namespace DeutschePost\Internetmarke\Integrations\Host;

use DeutschePost\Internetmarke\Abstracts\Singleton;
use DeutschePost\Internetmarke\Admin\Notice;
use DeutschePost\Internetmarke\Connectors\OneClickForApp\Connector as OneClickForAppConnector;
use DeutschePost\Internetmarke\Connectors\OneClickForApp\Response\AuthenticateUser as AuthenticateUserResponse;
use DeutschePost\Internetmarke\Connectors\OneClickForApp\Response\CheckoutShoppingCartPDF as CheckoutShoppingCartPDFResponse;
use DeutschePost\Internetmarke\Connectors\OneClickForApp\Response\CreateShopOrderId as CreateShopOrderIdResponse;
use DeutschePost\Internetmarke\Connectors\OneClickForApp\Response\RetrieveContractProducts;
use DeutschePost\Internetmarke\Connectors\OneClickForApp\Response\RetrieveContractProducts as RetrieveContractProductsResponse;
use DeutschePost\Internetmarke\Connectors\OneClickForApp\Response\RetrieveOrder as RetrieveOrderResponse;
use DeutschePost\Internetmarke\Connectors\OneClickForApp\Response\RetrievePageFormats as RetrievePageFormatsResponse;
use DeutschePost\Internetmarke\Connectors\OneClickForApp\Response\RetrievePreviewVoucherPDF as RetrievePreviewVoucherPDFResponse;
use DeutschePost\Internetmarke\Connectors\OneClickForApp\Response\RetrievePreviewVoucherPNG as RetrievePreviewVoucherPNGResponse;
use DeutschePost\Internetmarke\Connectors\OneClickForApp\Response\RetrievePrivateGallery as RetrievePrivateGalleryResponse;
use DeutschePost\Internetmarke\Connectors\OneClickForApp\Response\RetrievePublicGallery as RetrievePublicGalleryResponse;
use DeutschePost\Internetmarke\Models\Credentials\User as UserCredentialsModel;
use DeutschePost\Internetmarke\Models\Resources\ProductSales as ProductSalesResourceModel;
use DeutschePost\Internetmarke\Models\Resources\Shipment as ResourceShipmentModel;
use DeutschePost\Internetmarke\Repositories\Credentials\User as UserCredentialsRepository;
use DeutschePost\Internetmarke\Repositories\Resources\PageFormat as PageFormatResourceRepository;
use DeutschePost\Internetmarke\Repositories\Resources\ProductSales as ProductSalesResourceRepository;

class OneClickForApp extends Singleton
{
    /**
     * @var OneClickForAppConnector
     */
    protected $oneClickForAppConnector;

    /**
     * OneClickForApp constructor.
     *
     * @param OneClickForAppConnector $oneClickForAppConnector
     */
    public function __construct(OneClickForAppConnector $oneClickForAppConnector)
    {
        $this->oneClickForAppConnector = $oneClickForAppConnector;

        add_action('wcdpi_integration_oneclickforapp_update_contract_products', array($this, 'updateContractProducts'), 10);
        add_action('wcdpi_integration_oneclickforapp_update_page_formats',      array($this, 'updatePageFormats'),      10);
        add_filter('wcdpi_integration_oneclickforapp_purchase_label',           array($this, 'purchaseLabel'),          10, 1);
        add_filter('wcdpi_integration_oneclickforapp_preview_label',            array($this, 'previewLabel'),           10, 2);
        add_filter('wcdpi_integration_oneclickforapp_login',                    array($this, 'login'),                  10, 2);
        add_action('wcdpi_integration_oneclickforapp_update_wallet',            array($this, 'updateWallet'),           10);
    }

    /**
     * Authenticates the user against the remote webservice
     *
     * @param null|string $email
     * @param null|string $password
     *
     * @return AuthenticateUserResponse
     */
    public function authenticateUser($email = null, $password = null)
    {
        if ($email === null && $password === null) {
            /** @var UserCredentialsModel $userCredentials */
            $userCredentials = UserCredentialsRepository::instance()->get();

            $email = $userCredentials->getEmail();
            $password = $userCredentials->getPassword();
        }

        $args = array(
            'username' => $email,
            'password' => $password
        );

        /** @var AuthenticateUserResponse $authenticateUserResponse */
        $authenticateUserResponse = $this->oneClickForAppConnector->authenticateUser($args);

        if ($authenticateUserResponse->getInfoMessage()) {
            Notice::info($authenticateUserResponse->getInfoMessage());
        }

        $this->setUserToken($authenticateUserResponse->getUserToken());
        $this->setWalletBalance($authenticateUserResponse->getWalletBalance());

        return $authenticateUserResponse;
    }

    /**
     * Purchases a shipping label as a PDF
     *
     * @param ResourceShipmentModel $shipment
     * @param int                   $pageFormat
     * @param int                   $manifest
     * @param int                   $shippingList
     *
     * @return bool|CheckoutShoppingCartPDFResponse
     */
    public function checkoutShoppingCartPDF($shipment, $pageFormat, $manifest, $shippingList)
    {
        $args = array(
            'userToken'          => $this->getUserToken(),
            'pageFormatId'       => $pageFormat,
            'positions'          => $this->generatePositions($shipment),
            'total'              => $shipment->getPrice(),
            'shopOrderId'        => null,
            'ppl'                => null,
            'createManifest'     => $manifest,
            'createShippingList' => $shippingList
        );

        /** @var CheckoutShoppingCartPDFResponse $checkoutShoppingCartPDFResponse */
        $checkoutShoppingCartPDFResponse = $this->oneClickForAppConnector->checkoutShoppingCartPDF($args);

        return $checkoutShoppingCartPDFResponse;
    }

    /**
     * Purchases a shipping label as a PNG
     *
     * @param ResourceShipmentModel $shipment
     * @param int                   $manifest
     * @param int                   $shippingList
     *
     */
    public function checkoutShoppingCartPNG($shipment, $manifest, $shippingList) {}

    /**
     * Creates and retrieves a Shop Order ID
     *
     * @return CreateShopOrderIdResponse
     */
    public function createShopOrderId()
    {
        $args = array(
            'userToken' => $this->getUserToken()
        );

        /** @var CreateShopOrderIdResponse $createShopOrderIdResponse */
        $createShopOrderIdResponse = $this->oneClickForAppConnector->createShopOrderId($args);

        return $createShopOrderIdResponse;
    }

    /**
     * Retrieves the Portokasse contract products
     *
     * @return RetrieveContractProductsResponse
     */
    public function retrieveContractProducts()
    {
        $args = array(
            'userToken' => $this->getUserToken()
        );

        /** @var RetrieveContractProductsResponse $retrieveContractProductsResponse */
        $retrieveContractProductsResponse = $this->oneClickForAppConnector->retrieveContractProducts($args);

        return $retrieveContractProductsResponse;
    }

    /**
     * Retrieves a Shop Order by its ID
     *
     * @param int $shopOrderId
     *
     * @return RetrieveOrderResponse
     */
    public function retrieveOrder($shopOrderId)
    {
        $args = array(
            'userToken'   => $this->getUserToken(),
            'shopOrderId' => $shopOrderId
        );

        /** @var RetrieveOrderResponse $retrieveOrderResponse */
        $retrieveOrderResponse = $this->oneClickForAppConnector->retrieveOrder($args);

        return $retrieveOrderResponse;
    }

    /**
     * Retrieves the available page formats
     *
     * @return RetrievePageFormatsResponse
     */
    public function retrievePageFormats()
    {
        /** @var RetrievePageFormatsResponse $retrievePageFormatsResponse */
        $retrievePageFormatsResponse = $this->oneClickForAppConnector->retrievePageFormats();

        return $retrievePageFormatsResponse;
    }

    /**
     * Retrieves the preview voucher of a specific label in PDF format
     *
     * @param int    $productCode
     * @param string $voucherLayout
     * @param int    $pageFormatId
     * @param string $imageID
     *
     * @return RetrievePreviewVoucherPDFResponse
     */
    public function retrievePreviewVoucherPDF($productCode, $voucherLayout, $pageFormatId, $imageID)
    {
        $args = array(
            'productCode'   => $productCode,
            'voucherLayout' => $voucherLayout,
            'pageFormatId'  => $pageFormatId,
            'imageID'       => $imageID
        );

        /** @var RetrievePreviewVoucherPDFResponse $retrievePreviewVoucherPDFResponse */
        $retrievePreviewVoucherPDFResponse = $this->oneClickForAppConnector->retrievePreviewVoucherPDF($args);

        return $retrievePreviewVoucherPDFResponse;
    }

    /**
     * Retrieves the preview voucher of a specific label in PNG format
     *
     * @param int    $productCode
     * @param string $voucherLayout
     * @param int    $imageId
     *
     * @return RetrievePreviewVoucherPNGResponse
     */
    public function retrievePreviewVoucherPNG($productCode, $voucherLayout, $imageId)
    {
        $args = array(
            'productCode' => $productCode,
            'voucherLayout' => $voucherLayout,
            'imageID' => $imageId
        );

        /** @var RetrievePreviewVoucherPNGResponse $retrievePreviewVoucherPNGResponse */
        $retrievePreviewVoucherPNGResponse = $this->oneClickForAppConnector->retrievePreviewVoucherPNG($args);

        return $retrievePreviewVoucherPNGResponse;
    }

    /**
     * Retrieves the private gallery for the FrankingZone labels
     *
     * @return RetrievePrivateGalleryResponse
     */
    public function retrievePrivateGallery()
    {
        $args = array(
            'userToken' => $this->getUserToken()
        );

        /** @var RetrievePrivateGalleryResponse $retrievePrivateGalleryResponse */
        $retrievePrivateGalleryResponse = $this->oneClickForAppConnector->retrievePrivateGallery($args);

        return $retrievePrivateGalleryResponse;
    }

    /**
     * Retrieves the public gallery for the FrankingZone labels
     *
     * @return RetrievePublicGalleryResponse
     */
    public function retrievePublicGallery()
    {
        /** @var RetrievePublicGalleryResponse $retrievePublicGalleryResponse */
        $retrievePublicGalleryResponse = $this->oneClickForAppConnector->retrievePublicGallery();

        return $retrievePublicGalleryResponse;
    }

    /**
     * Performs an update of contract product prices
     *
     * @return boolean
     */
    public function updateContractProducts()
    {
        /** @var RetrieveContractProductsResponse $contractProducts */
        $contractProducts = $this->retrieveContractProducts();

        foreach ($contractProducts->getProducts() as $contractProduct) {
            if (property_exists($contractProduct, 'price')) {
                ProductSalesResourceRepository::instance()->updateBy(
                    array('product_code' => $contractProduct->productCode),
                    array('price' => $contractProduct->price)
                );
            }
        }

        update_option('wcdpi_update_contract_products_date', gmdate('Y-m-d H:i:s'), true);

        return true;
    }

    /**
     * @param string $email
     * @param string $password
     */
    public function login($email, $password)
    {
        $this->authenticateUser($email, $password);
    }

    /**
     * Performs a wallet update for the currently logged in user.
     */
    public function updateWallet()
    {
        $this->authenticateUser();
    }

    /**
     * @param ResourceShipmentModel $shipment
     *
     * @return bool
     */
    public function purchaseLabel($shipment)
    {
        // Get the options for the page format, manifest, and shipping list.
        $settingsOneClickForApp = (array)get_option('_wcdpi_settings_internetmarke_1c4a');
        $pageFormat = 1;
        $manifest = 0;
        $shippingList = 0;

        if (array_key_exists('_wcdpi_page_formats_default_label_format', $settingsOneClickForApp)) {
            $pageFormat = intval($settingsOneClickForApp['_wcdpi_page_formats_default_label_format']);
        }

        if (array_key_exists('_wcdpi_label_options_manifest', $settingsOneClickForApp)) {
            $manifest = intval($settingsOneClickForApp['_wcdpi_label_options_manifest']);
        }

        if (array_key_exists('_wcdpi_label_options_shipping_list', $settingsOneClickForApp)) {
            $shippingList = intval($settingsOneClickForApp['_wcdpi_label_options_shipping_list']);
        }

        return $this->checkoutShoppingCartPDF($shipment, $pageFormat, $manifest, $shippingList);
    }

    /**
     * @param int    $productCode
     * @param string $voucherLayout
     *
     * @return bool|string
     */
    public function previewLabel($productCode, $voucherLayout)
    {
        /** @var RetrievePreviewVoucherPNGResponse $voucherPreview */
        $voucherPreview = $this->retrievePreviewVoucherPNG($productCode, $voucherLayout, null);

        return $voucherPreview->getLink();
    }

    /**
     * @return bool
     */
    public function updatePageFormats()
    {
        /** @var RetrievePageFormatsResponse $pageFormats */
        $pageFormats = $this->retrievePageFormats();

        if (!$pageFormats instanceof RetrievePageFormatsResponse) {
            Notice::error(__('Page formats were not updated.', 'woo-dp-internetmarke'));
            return false;
        }

        PageFormatResourceRepository::instance()->truncate();

        foreach ($pageFormats->getPageFormat() as $pageFormat) {
            $args = array(
                'format_id'           => $pageFormat->id,
                'name'                => $pageFormat->name,
                'is_address_possible' => $pageFormat->isAddressPossible,
                'is_image_possible'   => $pageFormat->isImagePossible,
                'description'         => property_exists($pageFormat, 'description') ? $pageFormat->description : null,
                'page_type'           => $pageFormat->pageType,
                'page_layout'         => serialize($pageFormat->pageLayout)
            );

            PageFormatResourceRepository::instance()->insert($args);
        }

        update_option('wcdpi_update_page_format_date', gmdate('Y-m-d H:i:s'), true);

        Notice::success(__('Page formats updated successfully.', 'woo-dp-internetmarke'));

        return true;
    }

    /**
     * @param ResourceShipmentModel $shipment
     *
     * @return array
     */
    private function generatePositions($shipment)
    {
        $address = array();

        /** @var ProductSalesResourceModel $productSales */
        $productSales = ProductSalesResourceRepository::instance()->findBy(
            array(
                'prodws_id' => $shipment->getProdwsId()
            )
        );

        // Sender
        if ($shipment->getSenderCompanyName()) {
            $address['sender']['name'] = array(
                'companyName' => array(
                    'company' => $shipment->getSenderCompanyName(),
                    'personName' => array(
                        'firstname' => $shipment->getSenderFirstName(),
                        'lastname' => $shipment->getSenderLastName()
                    )
                )
            );
        }

        else {
            $address['sender']['name'] = array(
                'personName' => array(
                    'firstname' => $shipment->getSenderFirstName(),
                    'lastname' => $shipment->getSenderLastName()
                )
            );
        }

        $address['sender']['address'] = array(
            'street'     => $shipment->getSenderStreet(),
            'houseNo'    => $shipment->getSenderHouseNumber(),
            'additional' => $shipment->getSenderAdditional(),
            'zip'        => $shipment->getSenderZip(),
            'city'       => $shipment->getSenderCity(),
            'country'    => $shipment->getSenderCountryAlpha3()
        );

        // Recipient
        if ($shipment->getRecipientCompanyName()) {
            $address['receiver']['name'] = array(
                'companyName' => array(
                    'company' => $shipment->getRecipientCompanyName(),
                    'personName' => array(
                        'firstname' => $shipment->getRecipientFirstName(),
                        'lastname' => $shipment->getRecipientLastName()
                    )
                )
            );
        }

        else {
            $address['receiver']['name'] = array(
                'personName' => array(
                    'firstname' => $shipment->getRecipientFirstName(),
                    'lastname' => $shipment->getRecipientLastName()
                )
            );
        }

        $address['receiver']['address'] = array(
            'street'     => $shipment->getRecipientStreet(),
            'houseNo'    => $shipment->getRecipientHouseNumber(),
            'additional' => $shipment->getRecipientAdditional(),
            'zip'        => $shipment->getRecipientZip(),
            'city'       => $shipment->getRecipientCity(),
            'country'    => $shipment->getRecipientCountryAlpha3()
        );

        return array(
            'productCode' => $productSales->getProductCode(),
            //'imageId' => 1335564250, // for custom image (FrankingZone)
            'voucherLayout' => 'AddressZone',
            'position' => array(
                'labelX' => 1,
                'labelY' => 1,
                'page' => 1
            ),
            'address' => $address
        );
    }

    /**
     * Retrieves the Merchant Partner ID
     * 
     * @return string
     */
    public function getPartnerId()
    {
        return $this->oneClickForAppConnector->getPartnerId();
    }

    /**
     * Sets the Merchant Partner ID
     *
     * @var int $partnerId
     */
    public function setPartnerId($partnerId)
    {
        $this->oneClickForAppConnector->setPartnerId($partnerId);
    }

    /**
     * Retrieves the Merchant Key Phase
     * 
     * @return int
     */
    public function getKeyPhase()
    {
        return $this->oneClickForAppConnector->getKeyPhase();
    }

    /**
     * Sets the Merchant Key Phase
     *
     * @var int $keyPhase
     */
    public function setKeyPhase($keyPhase)
    {
        $this->oneClickForAppConnector->setKeyPhase($keyPhase);
    }

    /**
     * Retrieves the Merchant Marketplace Key
     * 
     * @return string
     */
    public function getMarketplaceKey()
    {
        return $this->oneClickForAppConnector->getMarketplaceKey();
    }

    /**
     * Sets the Merchant Marketplace Key
     *
     * @var string $marketplaceKey
     */
    public function setMarketplaceKey($marketplaceKey)
    {
        $this->oneClickForAppConnector->setMarketplaceKey($marketplaceKey);
    }

    /**
     * Retrieves the User Authentication Token
     *
     * @return string
     */
    public function getUserToken()
    {
        if ($userToken = get_transient('wcdpi_oneclickforapp_user_token')) {
            return $userToken;
        }

        $this->authenticateUser();

        return get_transient('wcdpi_oneclickforapp_user_token');
    }

    /**
     * Stores the User Authentication Token inside a transient
     * 
     * @param string  $value      The new User Token
     * @param integer $expiration Transient expiration (in seconds)
     */
    private function setUserToken($value, $expiration = 86400)
    {
        set_transient('wcdpi_oneclickforapp_user_token', $value, $expiration);
    }

    /**
     * Retrieves the User Wallet Balance
     *
     * @param bool $formatted
     *
     * @return int
     */
    public function getWalletBalance($formatted = false)
    {
        $walletBalance = get_option('wcdpi_oneclickforapp_wallet_balance');

        if (!$walletBalance) {
            $this->authenticateUser();
            $walletBalance = get_option('wcdpi_oneclickforapp_wallet_balance');
        }

        if ($formatted) {
            return wcdpi_format_price($walletBalance / 100);
        }

        return $walletBalance;
    }

    /**
     * Stores the User Wallet Balance
     * 
     * @param integer $value The new Wallet Balance
     */
    private function setWalletBalance($value)
    {
        update_option('wcdpi_oneclickforapp_wallet_balance', $value);
    }
}