<?php

namespace DeutschePost\Internetmarke\Integrations\Host;

use DeutschePost\Internetmarke\Abstracts\Singleton;
use DeutschePost\Internetmarke\Connectors\OneClickForRefund\Connector as OneClickForRefundConnector;
use DeutschePost\Internetmarke\Connectors\OneClickForRefund\Response\AuthenticateUser as AuthenticateUserResponse;
use DeutschePost\Internetmarke\Connectors\OneClickForRefund\Response\CreateRetoureId as CreateRetoureIdResponse;
use DeutschePost\Internetmarke\Models\Credentials\User as UserCredentialsModel;
use DeutschePost\Internetmarke\Models\Resources\Shipment as ResourceShipmentModel;
use DeutschePost\Internetmarke\Repositories\Credentials\User as UserCredentialsRepository;

class OneClickForRefund extends Singleton
{
    /**
     * @var OneClickForRefundConnector
     */
    protected $oneClickForRefundConnector;

    /**
     * OneClickForRefund constructor.
     * 
     * @param OneClickForRefundConnector $oneClickForRefundConnector
     */
    public function __construct(OneClickForRefundConnector $oneClickForRefundConnector)
    {
        $this->oneClickForRefundConnector = $oneClickForRefundConnector;

        add_filter('wcdpi_integration_oneclickforrefund_refund_label', array($this, 'refundLabel'), 10, 1);
    }

    /**
     * Authenticates the user against the remote webservice
     *
     * @return AuthenticateUserResponse
     */
    protected function authenticateUser()
    {
        /** @var UserCredentialsModel $userCredentials */
        $userCredentials = UserCredentialsRepository::instance()->get();

        /** @var AuthenticateUserResponse $authentication */
        $authentication = $this->oneClickForRefundConnector->authenticateUser(
            array(
                'username' => $userCredentials->getEmail(),
                'password' => $userCredentials->getPassword()
            )
        );

        $this->setUserToken($authentication->getUserToken());

        return $authentication;
    }

    /**
     * Creates a retoure ID used for refunding labels.
     *
     * @return CreateRetoureIdResponse
     */
    protected function createRetoureId()
    {
        /** @var CreateRetoureIdResponse $result */
        $result = $this->oneClickForRefundConnector->createRetoureId();

        return $result;
    }

    /**
     * @param ResourceShipmentModel $shipment
     *
     * @return bool
     */
    protected function retoureVouchers($shipment)
    {
        $args = array(
            'userToken'     => $this->getUserToken(),
            'shopRetoureId' => $this->getRetoureId(),
            'shoppingCart'  => $this->generatePositions($shipment)
        );

        return $this->oneClickForRefundConnector->retoureVouchers($args);
    }

    protected function retrieveRetoureState()
    {
        // TODO Implement retrieveRetoureState()
    }

    /**
     * @param ResourceShipmentModel $shipment
     *
     * @return bool
     */
    public function refundLabel($shipment)
    {
        return $this->retoureVouchers($shipment);
    }

    /**
     * @param ResourceShipmentModel $shipment
     *
     * @return array
     */
    public function generatePositions($shipment)
    {
        return array(
            'shopOrderId' => $shipment->getShopOrderId()
        );
    }

    /**
     * Retrieves the User Authentication Token
     *
     * @return string
     */
    public function getUserToken()
    {
        if ($userToken = get_transient('wcdpi_oneclickforrefund_user_token')) {
            return $userToken;
        }

        $this->authenticateUser();

        return get_transient('wcdpi_oneclickforrefund_user_token');
    }

    /**
     * Stores the User Authentication Token inside a transient
     *
     * @param string  $value      The new User Token
     * @param integer $expiration Transient expiration (in seconds)
     */
    private function setUserToken($value, $expiration = 86400)
    {
        set_transient('wcdpi_oneclickforrefund_user_token', $value, $expiration);
    }

    /**
     * Retrieves a new retoure ID from the remote webservice
     *
     * @return string
     */
    public function getRetoureId()
    {
        $response = $this->createRetoureId();

        return $response->getShopRetoureId();
    }
}