<?php

namespace DeutschePost\Internetmarke\Integrations\Host;

use DeutschePost\Internetmarke\Abstracts\Singleton;
use DeutschePost\Internetmarke\Connectors\PacketPlus\Connector as PacketPlusConnector;
use DeutschePost\Internetmarke\Connectors\PacketPlus\Response\GetAccessToken as GetAccessTokenResponse;
use DeutschePost\Internetmarke\Connectors\PacketPlus\Response\GetTrackings as GetTrackingsResponse;
use DeutschePost\Internetmarke\Models\Credentials\User as UserCredentialsModel;
use DeutschePost\Internetmarke\Models\Resources\ProductSales as ProductSalesResourceModel;
use DeutschePost\Internetmarke\Models\Resources\Shipment as ResourceShipmentModel;
use DeutschePost\Internetmarke\Models\Resources\ShipmentItem as ShipmentItemResourceModel;
use DeutschePost\Internetmarke\Repositories\Credentials\User as UserCredentialsRepository;
use DeutschePost\Internetmarke\Repositories\Resources\ProductSales;
use DeutschePost\Internetmarke\Repositories\Resources\ShipmentItem as ShipmentItemResourceRepository;

class PacketPlus extends Singleton
{
    /**
     * @var PacketPlusConnector
     */
    protected $packetPlusConnector;

    /**
     * PacketPlus constructor.
     *
     * @param PacketPlusConnector $packetPlusConnector
     */
    public function __construct(PacketPlusConnector $packetPlusConnector)
    {
        $this->packetPlusConnector = $packetPlusConnector;

        add_filter('wcdpi_integration_packetplus_purchase_label', array($this, 'purchaseLabel'), 10, 2);
        add_filter('wcdpi_integration_packetplus_preview_label',  array($this, 'previewLabel'),  10, 1);
        add_filter('wcdpi_integration_packetplus_track_label',    array($this, 'trackLabel'),    10, 1);
    }

    /**
     * Retrieves the web service authentication token and stores it inside a transient.
     */
    protected function authenticateUser()
    {
        // Development
        if (defined('PACKETPLUS_QA_PORTOKASSE_EMAIL') && defined('PACKETPLUS_QA_PORTOKASSE_PASSWORD')) {
            $username = PACKETPLUS_QA_PORTOKASSE_EMAIL;
            $password = PACKETPLUS_QA_PORTOKASSE_PASSWORD;
            $expiration = 240;
        }

        // Productive
        else {
            /** @var UserCredentialsModel $userCredentials */
            $userCredentials = UserCredentialsRepository::instance()->get();

            $username = $userCredentials->getEmail();
            $password = $userCredentials->getPassword();
            $expiration = 18000;
        }

        /** @var GetAccessTokenResponse $authentication */
        $authentication = $this->packetPlusConnector->getAccessToken(
            array(
                'username' => $username,
                'password' => $password
            )
        );

        $this->setUserToken($authentication->getUserToken(), $expiration);
    }

    /**
     * @param ResourceShipmentModel $shipment
     *
     * @return bool
     */
    protected function createOrder($shipment)
    {
        $args = array(
            'token' => $this->getUserToken(),
            'data' => $this->generatePositions($shipment)
        );

        $result = $this->packetPlusConnector->createOrder($args);

        return $result;
    }

    /**
     * @param int $id
     *
     * @return mixed
     */
    protected function getItemLabel($id)
    {
        $args = array(
            'token' => $this->getUserToken(),
            'itemId' => $id
        );

        $result = $this->packetPlusConnector->getItemLabel($args);

        return $result;
    }

    /**
     * @param $barcode
     *
     * @return mixed
     */
    protected function getTrackings($barcode)
    {
        $args = array(
            'token' => $this->getUserToken(),
            'barcode' => $barcode
        );

        $result = $this->packetPlusConnector->getTrackings($args);

        return $result;
    }

    /**
     * @param ResourceShipmentModel $shipment
     *
     * @return bool
     */
    public function purchaseLabel($shipment)
    {
        $result = $this->createOrder($shipment);

        return $result;
    }

    /**
     * @param string $prodwsId
     *
     * @return string
     */
    public function previewLabel($prodwsId)
    {
        return WCDPI_IMG_URL . 'packet_plus_previews/' . $prodwsId . '.png';
    }

    /**
     * @param string $barcode
     *
     * @return GetTrackingsResponse
     */
    public function trackLabel($barcode)
    {
        $result = $this->getTrackings($barcode);

        return $result;
    }

    /**
     * @param int $id
     *
     * @return mixed
     */
    public function getLabel($id)
    {
        $label = $this->getItemLabel($id);

        return $label;
    }

    /**
     * @param ResourceShipmentModel $shipment
     *
     * @return array
     */
    private function generatePositions($shipment)
    {
        $contents                = array();
        $settingsPacketPlus      = (array)get_option('_wcdpi_settings_packet_plus_international');
        $settingsGeneral         = (array)get_option('_wcdpi_settings_general');
        $ekp                     = '0000010000';
        $shipmentNatureType      = 'SALE_GOODS';
        $lineItemOrigin          = 'DE';
        $serviceLevel            = 'PRIORITY';
        $contentPieceIndexNumber = 0;
        $shipmentAmount          = 0;
        $shipmentGrossWeight     = 0;
        $storeCurrency           = get_woocommerce_currency();
        $contactName             = '';
        $senderName              = $shipment->getSenderFirstName() . ' ' . $shipment->getSenderLastName();
        $recipientName           = $shipment->getRecipientFirstName() . ' ' . $shipment->getRecipientLastName();

        /** @var ProductSalesResourceModel $productSales */
        $productSales = ProductSales::instance()->findBy(
            array(
                'prodws_id' => $shipment->getProdwsId()
            )
        );

        if (array_key_exists('_wcdpi_packet_plus_international_ekp', $settingsPacketPlus)) {
            $ekp = $settingsPacketPlus['_wcdpi_packet_plus_international_ekp'];
        }

        if (array_key_exists('_wcdpi_packet_plus_international_default_shipment_nature_type', $settingsPacketPlus)) {
            $shipmentNatureType = $settingsPacketPlus['_wcdpi_packet_plus_international_default_shipment_nature_type'];
        }

        if (array_key_exists('_wcdpi_packet_plus_international_default_line_item_origin', $settingsPacketPlus)) {
            $lineItemOrigin = $settingsPacketPlus['_wcdpi_packet_plus_international_default_line_item_origin'];
        }

        if (array_key_exists('_wcdpi_packet_plus_international_service_level', $settingsPacketPlus)) {
            $serviceLevel = $settingsPacketPlus['_wcdpi_packet_plus_international_service_level'];
        }

        if (array_key_exists('_wcdpi_default_sender_contact_name', $settingsGeneral)) {
            $contactName = $settingsGeneral['_wcdpi_default_sender_contact_name'];
        }

        /** @var ShipmentItemResourceModel[] $shipmentItems */
        $shipmentItems = ShipmentItemResourceRepository::instance()->findAllBy(
            array(
                'shipment_id' => $shipment->getId()
            )
        );

        if (!$shipmentItems) {

        }

        /** @var ShipmentItemResourceModel $item */
        foreach ($shipmentItems as $item) {
            if (!$item->getProductQuantity()) {
                continue;
            }

            $hsCode               = $item->getProductHsCode();
            $customsDeclaration   = $item->getProductCustomsDeclaration();
            $weight               = floatval($item->getProductWeight());
            $countryOfManufacture = $item->getProductOrigin() ? $item->getProductOrigin() : $lineItemOrigin;
            $qtySelected          = intval($item->getProductQuantity());
            $price                = floatval($item->getProductPrice());
            $subtotalPrice        = $qtySelected * $price;
            $subtotalWeight       = $qtySelected * $weight;

            $shipmentAmount      += $subtotalPrice;
            $shipmentGrossWeight += $subtotalWeight;

            $contentPiece = array(
                'contentPieceIndexNumber' => $contentPieceIndexNumber++,
                'contentPieceHsCode'      => $hsCode,
                'contentPieceDescription' => $customsDeclaration,
                'contentPieceNetweight'   => $subtotalWeight,
                'contentPieceOrigin'      => $countryOfManufacture,
                'contentPieceAmount'      => $qtySelected,
                'contentPieceValue'       => number_format($subtotalPrice, 2, '.', '')
            );

            $contents[] = apply_filters('internetmarke_content_piece', $contentPiece);
        }

        $positions = array(
            'customerEkp' => $ekp,
            'orderStatus' => 'FINALIZE',
            'paperwork' => array(
                'contactName' => $contactName,
                'jobReference' => null,
                'pickupType' => 'CUSTOMER_DROP_OFF',
                'awbCopyCount' => 1
            ),
            'items' => array(
                array(
                    'id'                  => null,
                    'product'             => $productSales->getProductCode(),
                    'serviceLevel'        => $serviceLevel,
                    'custRef'             => null,
                    'custRef2'            => null,
                    'recipient'           => $recipientName,
                    'recipientPhone'      => str_replace(' ', '', $shipment->getRecipientPhone()),
                    'recipientFax'        => null,
                    'recipientEmail'      => $shipment->getRecipientEmail(),
                    'addressLine1'        => $shipment->getRecipientStreet() . ' ' . $shipment->getRecipientHouseNumber(),
                    'addressLine2'        => $shipment->getRecipientAdditional(),
                    'addressLine3'        => $shipment->getRecipientCompanyName(),
                    'city'                => $shipment->getRecipientCity(),
                    'state'               => $shipment->getRecipientState(),
                    'postalCode'          => $shipment->getRecipientZip(),
                    'destinationCountry'  => $shipment->getRecipientCountryAlpha2(),
                    'returnItemWanted'    => false,
                    'shipmentAmount'      => number_format($shipmentAmount, 2, '.', ''),
                    'shipmentCurrency'    => $storeCurrency,
                    'shipmentGrossWeight' => $shipmentGrossWeight,
                    'senderName'          => $senderName,
                    'senderAddressLine1'  => $shipment->getSenderStreet() . ' ' . $shipment->getSenderHouseNumber(),
                    'senderAddressLine2'  => $shipment->getSenderAdditional(),
                    'senderCountry'       => $shipment->getSenderCountryAlpha2(),
                    'senderCity'          => $shipment->getSenderCity(),
                    'senderPostalCode'    => $shipment->getSenderZip(),
                    'senderPhone'         => str_replace(' ', '', $shipment->getSenderPhone()),
                    'senderEmail'         => $shipment->getSenderEmail(),
                    'shipmentNaturetype'  => $shipmentNatureType,
                    'contents'            => array()
                )
            )
        );

        // Only add the contents if the recipient is outside of the EU
        if (!in_array($shipment->getRecipientCountryAlpha2(), wcdpi_get_eu_countries())) {
            $positions['items'][0]['contents'] = $contents;
        }

        return $positions;
    }

    /**
     * @param string $value
     * @param int    $expiration
     */
    public function setUserToken($value, $expiration = 18000)
    {
        set_transient('wcdpi_packetplus_user_token', $value, $expiration);
    }

    /**
     * @return string
     */
    public function getUserToken()
    {
        if ($userToken = get_transient('wcdpi_packetplus_user_token')) {
            return $userToken;
        }

        $this->authenticateUser();

        return get_transient('wcdpi_packetplus_user_token');
    }
}