<?php

namespace DeutschePost\Internetmarke\Integrations\Host;

use DeutschePost\Internetmarke\Abstracts\Singleton;
use DeutschePost\Internetmarke\Admin\Notice;
use DeutschePost\Internetmarke\Connectors\ProdWS\Connector as ProdWSConnector;
use DeutschePost\Internetmarke\Connectors\ProdWS\Response\GetProductList as GetProductListResponse;
use DeutschePost\Internetmarke\Models\Credentials\Partner as PartnerCredentialsModel;
use DeutschePost\Internetmarke\Repositories\Credentials\Partner as PartnerCredentialsRepository;
use DeutschePost\Internetmarke\Repositories\Resources\ProductAdditional as ProductAdditionalResourceRepository;
use DeutschePost\Internetmarke\Repositories\Resources\ProductBasic as ProductBasicResourceRepository;
use DeutschePost\Internetmarke\Repositories\Resources\ProductSales as ProductSalesResourceRepository;

defined('ABSPATH') || exit;

class ProdWS extends Singleton
{
    /**
     * @var ProdWSConnector
     */
    protected $prodWSConnector;

    /**
     * ProdWS constructor.
     * 
     * @param ProdWSConnector $prodWSConnector
     */
    public function __construct(ProdWSConnector $prodWSConnector)
    {
        $this->prodWSConnector = $prodWSConnector;

        add_action('wcdpi_integration_prodws_update_product_list', array($this, 'updateProductList'));
    }

    protected function getCatalog()
    {
        // TODO implement getCatalog()
    }

    protected function getCatalogList()
    {
        // TODO Implement getCatalogList()
    }

    protected function getProductChangeInformation()
    {
        // TODO Implement getProductChangeInformation()
    }

    /**
     * Retrieves the complete Product List
     * 
     * @return GetProductListResponse
     */
    protected function getProductList()
    {
        /** @var PartnerCredentialsModel $partnerCredentials */
        $partnerCredentials = PartnerCredentialsRepository::instance()->get();

        return $this->prodWSConnector->getProductList(
            array(
                'mandantID'         => $partnerCredentials->get('ProdWs', 'MandantId'),
                'subMandantID'      => null,
                'timestamp'         => null,
                'dedicatedProducts' => true,
                'responseMode'      => 0,
                'onlyChanges'       => false,
                'referenceDate'     => null,
                'shortList'         => false
            )
        );
    }

    protected function getProductVersionsList()
    {
        // TODO Implement getProductVersionsList()
    }

    protected function registerEMailAddress()
    {
        // TODO Implement registerEMailAddress()
    }

    protected function registerNotification()
    {
        // TODO Implement registerNotification()
    }

    /**
     * @return bool
     */
    public function updateProductList()
    {
        $productList = $this->getProductList();

        if (!$productList instanceof GetProductListResponse) {
            Notice::error(__('Product list was not updated.', 'woo-dp-internetmarke'));
            return false;
        }

        ProductBasicResourceRepository::instance()->truncate();
        ProductAdditionalResourceRepository::instance()->truncate();
        ProductSalesResourceRepository::instance()->truncate();

        foreach ($productList->getBasicProductList() as $basicProduct) {
            $extendedIdentifier = $basicProduct->extendedIdentifier;
            $args = array(
                'prodws_id'   => $extendedIdentifier->{'ProdWS-ID'},
                'name'        => $extendedIdentifier->name,
                'version'     => $extendedIdentifier->version,
                'description' => (isset($extendedIdentifier->description)) ? $extendedIdentifier->description : null,
                'destination' => $extendedIdentifier->destination,
                'valid_from'  => $extendedIdentifier->validFrom,
                'valid_to'    => (isset($extendedIdentifier->validTo) ? $extendedIdentifier->validTo : null),
                'price'       => wcdpi_euros_to_cents($basicProduct->priceDefinition->grossPrice->value),
            );

            ProductBasicResourceRepository::instance()->insert($args);
        }

        foreach ($productList->getAdditionalProductList() as $additionalProduct) {
            $extendedIdentifier = $additionalProduct->extendedIdentifier;

            $args = array(
                'prodws_id'   => $extendedIdentifier->{'ProdWS-ID'},
                'name'        => $extendedIdentifier->name,
                'version'     => $extendedIdentifier->version,
                'annotation'  => (isset($extendedIdentifier->annotation)) ? $extendedIdentifier->annotation : null,
                'destination' => $extendedIdentifier->destination,
                'valid_from'  => $extendedIdentifier->validFrom,
                'valid_to'    => (isset($extendedIdentifier->validTo) ? $extendedIdentifier->validTo : null),
                'price'       => wcdpi_euros_to_cents($additionalProduct->priceDefinition->grossPrice->value),
            );

            ProductAdditionalResourceRepository::instance()->insert($args);
        }

        foreach ($productList->getSalesProductList() as $salesProduct) {
            $extendedIdentifier = $salesProduct->extendedIdentifier;
            $externIdentifier   = $extendedIdentifier->externIdentifier;
            $lengthMin          = null;
            $lengthMax          = null;
            $lengthUnit         = null;
            $widthMin           = null;
            $widthMax           = null;
            $widthUnit          = null;
            $heightMin          = null;
            $heightMax          = null;
            $heightUnit         = null;
            $weightMin          = null;
            $weightMax          = null;
            $weightUnit         = null;

            if (property_exists($salesProduct, 'dimensionList')) {
                $dimensions = $salesProduct->dimensionList;

                // Length
                if (property_exists($dimensions, 'length')) {
                    $l = $dimensions->length;

                    $lengthMin  = property_exists($l, 'minValue') ? $l->minValue : null;
                    $lengthMax  = property_exists($l, 'maxValue') ? $l->maxValue : null;
                    $lengthUnit = property_exists($l, 'unit')     ? $l->unit     : null;
                }

                // Width
                if (property_exists($dimensions, 'width')) {
                    $w = $dimensions->width;

                    $widthMin  = property_exists($w, 'minValue') ? $w->minValue : null;
                    $widthMax  = property_exists($w, 'maxValue') ? $w->maxValue : null;
                    $widthUnit = property_exists($w, 'unit')     ? $w->unit     : null;
                }

                // Height
                if (property_exists($dimensions, 'height')) {
                    $h = $dimensions->height;

                    $heightMin  = property_exists($h, 'minValue') ? $h->minValue : null;
                    $heightMax  = property_exists($h, 'maxValue') ? $h->maxValue : null;
                    $heightUnit = property_exists($h, 'unit')     ? $h->unit     : null;
                }
            }

            // Weight
            if (property_exists($salesProduct, 'weight')) {
                $w = $salesProduct->weight;

                $weightMin  = (property_exists($w, 'minValue')) ? $w->minValue : null;
                $weightMax  = (property_exists($w, 'maxValue')) ? $w->maxValue : null;
                $weightUnit = (property_exists($w, 'unit'))     ? $w->unit     : null;
            }

            $args = array(
                'prodws_id'        => $extendedIdentifier->{'ProdWS-ID'},
                'product_code'     => $externIdentifier->id,
                'name'             => $externIdentifier->name,
                'annotation'       => property_exists($extendedIdentifier, 'annotation') ? $extendedIdentifier->annotation : null,
                'destination'      => $extendedIdentifier->destination,
                'version'          => $extendedIdentifier->version,
                'valid_from'       => $extendedIdentifier->validFrom,
                'valid_to'         => property_exists($extendedIdentifier, 'validTo') ? $extendedIdentifier->validTo : null,
                'price'            => wcdpi_euros_to_cents($salesProduct->priceDefinition->price->calculatedGrossPrice->value),
                'ppl'              => $externIdentifier->actualPPLVersion,
                'information_text' => $this->getInformationText((array)$salesProduct->stampTypeList->stampType),
                'length_min'       => $lengthMin,
                'length_max'       => $lengthMax,
                'length_unit'      => $lengthUnit,
                'width_min'        => $widthMin,
                'width_max'        => $widthMax,
                'width_unit'       => $widthUnit,
                'height_min'       => $heightMin,
                'height_max'       => $heightMax,
                'height_unit'      => $heightUnit,
                'weight_min'       => $weightMin,
                'weight_max'       => $weightMax,
                'weight_unit'      => $weightUnit
            );

            ProductSalesResourceRepository::instance()->insert($args);
        }

        update_option('wcdpi_update_product_list_date', gmdate('Y-m-d H:i:s'), true);
        Notice::success(__('Product list updated successfully.', 'woo-dp-internetmarke'));
        return true;
    }

    /**
     * @param array $stampType
     *
     * @return string
     */
    private function getInformationText($stampType)
    {
        $informationText = '';

        foreach ($stampType as $stamp) {
            if (property_exists($stamp, 'name') && $stamp->name == 'Plugin') {
                foreach ($stamp->propertyList as $properties) {
                    foreach ($properties as $property) {
                        if ($property->name == 'InformationText') {
                            $informationText = $property->propertyValue->alphanumericValue->fixValue;
                        }
                    }
                }
            }
        }

        return $informationText;
    }
}