<?php

namespace DeutschePost\Internetmarke\Integrations\WooCommerce;

use DeutschePost\Internetmarke\Enums\ShippingRegion as ShippingRegionEnum;
use WC_Order_Item_Product;
use WC_Product_Simple;
use WP_Query;
use function WC;

defined('ABSPATH') || exit;

class Orders
{
    /**
     * Orders constructor.
     */
    public function __construct()
    {
        add_action('restrict_manage_posts',               array($this, 'addDestinationFilter'), 10, 2);
        add_action('pre_get_posts',                       array($this, 'handleDestinationFilter'));
        add_filter('bulk_actions-edit-shop_order',        array($this, 'addCreateDeutschePostShipmentAction'));
        add_filter('handle_bulk_actions-edit-shop_order', array($this, 'handleCreateDeutschePostShipmentAction'), 10, 3);
    }

    /**
     * Adds a "Destination" filter to the WooCommerce Orders list.
     *
     * @see https://developer.wordpress.org/reference/hooks/restrict_manage_posts/
     *
     * @param string $postType The post type slug
     * @param string $which    The location of the extra table nav markup
     */
    public function addDestinationFilter($postType, $which)
    {
        if ($postType !== 'shop_order') {
            return;
        }

        if ($which !== 'top') {
            return;
        }

        $destination = $this->getOrderDestination();

        echo '<select id="filter-by-shipping-region" name="wcdpi_order_destination">';

        echo '<option value=""></option>';

        echo sprintf(
            '<option value="%s" %s>%s</option>',
            ShippingRegionEnum::ALL,
            selected($destination, ShippingRegionEnum::ALL, false),
            __('All', 'woo-dp-internetmarke')
        );

        echo sprintf(
            '<option value="%s" %s>%s</option>',
            ShippingRegionEnum::DOMESTIC,
            selected($destination, ShippingRegionEnum::DOMESTIC, false),
            WC()->countries->countries['DE']
        );

        echo sprintf(
            '<option value="%s" %s>%s</option>',
            ShippingRegionEnum::EUROPEAN_UNION,
            selected($destination, ShippingRegionEnum::EUROPEAN_UNION, false),
            __('European union', 'woo-dp-internetmarke')
        );

        echo sprintf(
            '<option value="%s" %s>%s</option>',
            ShippingRegionEnum::REST_OF_WORLD,
            selected($destination, ShippingRegionEnum::REST_OF_WORLD, false),
            __('Rest of world', 'woo-dp-internetmarke')
        );

        echo '</select>';
    }

    /**
     * Handles the "Destination" filter request.
     *
     * @see https://developer.wordpress.org/reference/hooks/pre_get_posts/
     *
     * @param WP_Query $query The WP_Query instance (passed by reference)
     */
    public function handleDestinationFilter($query)
    {
        if (!$query->is_admin || !isset($_GET['wcdpi_order_destination']) || $_GET['post_type'] !== 'shop_order') {
            return;
        }

        $destinationFilter = $_GET['wcdpi_order_destination'];

        switch ($destinationFilter) {
            case ShippingRegionEnum::DOMESTIC:
                // All orders whose shipping country Germany
                $metaQuery = array(
                    array(
                        'key'   => '_shipping_country',
                        'value' => 'DE'
                    )
                );
                break;
            case ShippingRegionEnum::EUROPEAN_UNION:
                // All orders whose shipping country is inside the EU, but not Germany
                $metaQuery = array(
                    'relation'  => 'AND',
                    'eu_clause' => array(
                        'key'     => '_shipping_country',
                        'value'   => wcdpi_get_eu_countries(),
                        'compare' => 'IN'
                    ),
                    'de_clause' => array(
                        'key'     => '_shipping_country',
                        'value'   => array('DE'),
                        'compare' => 'NOT IN'
                    )
                );
                break;
            case ShippingRegionEnum::REST_OF_WORLD:
                // All orders whose shipping country is outside the EU
                $metaQuery = array(
                    array(
                        'key'     => '_shipping_country',
                        'value'   => wcdpi_get_eu_countries(),
                        'compare' => 'NOT IN'
                    )
                );
                break;
            default:
                $metaQuery = false;
                break;
        }

        if ($metaQuery) {
            $query->set('meta_query', $metaQuery);
        }
    }

    /**
     * Adds the "Create Deutsche Post Shipment" bulk action to the WooCommerce Orders list.
     *
     * @see https://developer.wordpress.org/reference/hooks/bulk_actions-this-screen-id/
     *
     * @param array $actions An array of the available bulk actions
     *
     * @return array
     */
    public function addCreateDeutschePostShipmentAction($actions)
    {
        $actions['wcdpi_bulk_create_shipment'] = __('Internetmarke - Create a shipment', 'woo-dp-internetmarke');

        return $actions;
    }

    /**
     * Handles the "Create Deutsche Post Shipment" bulk action.
     *
     * @see https://developer.wordpress.org/reference/hooks/handle_bulk_actions-screen/
     *
     * @param string $redirectTo The redirect URL
     * @param string $doAction   The action being taken
     * @param array  $items      The items to take the action on
     *
     * @return string
     */

    public function handleCreateDeutschePostShipmentAction($redirectTo, $doAction, $items)
    {
        if ($doAction !== 'wcdpi_bulk_create_shipment') {
            return $redirectTo;
        }

        if (empty($items)) {
            return $redirectTo;
        }

        setcookie('internetmarke_shipments_bulk_create', null, -1, '/');

        setcookie(
            'internetmarke_shipments_bulk_create',
            json_encode(
                array(
                    'event'       => 'create_bulk_order',
                    'destination' => $this->getOrderDestination(),
                    'orders'      => $items
                )
            ),
            time() + 86400,
            '/'
        );

        return admin_url('admin.php?page=internetmarke_shipments_bulk_create');
    }

    /**
     * Retrieves the order destination from the current request.
     *
     * @return string
     */
    private function getOrderDestination()
    {
        $destination = '';

        if (isset($_REQUEST['wcdpi_order_destination'])) {
            switch ($_REQUEST['wcdpi_order_destination']) {
                case ShippingRegionEnum::ALL:
                    $destination = ShippingRegionEnum::ALL;
                    break;
                case ShippingRegionEnum::EUROPEAN_UNION:
                    $destination = ShippingRegionEnum::EUROPEAN_UNION;
                    break;
                case ShippingRegionEnum::REST_OF_WORLD:
                    $destination = ShippingRegionEnum::REST_OF_WORLD;
                    break;
                case ShippingRegionEnum::DOMESTIC:
                    $destination = ShippingRegionEnum::DOMESTIC;
                    break;
                default:
                    $destination = '';
            }
        }

        return $destination;
    }
}