<?php

namespace DeutschePost\Internetmarke;

use SoapFault;
use DeutschePost\Internetmarke\Abstracts\Singleton;
use DeutschePost\Internetmarke\Admin\Notice;
use DeutschePost\Internetmarke\Connectors\REST\FORKLane;
use DeutschePost\Internetmarke\Services\LabelService;
use DeutschePost\Internetmarke\Services\ShipmentService;
use DeutschePost\Internetmarke\Services\ShipmentItemService;

defined('ABSPATH') || exit;

final class Internetmarke extends Singleton
{
    /**
     * @var Logger
     */
    protected static $logger;

    /**
     * @var LabelService
     */
    public $labelService;

    /**
     * @var ShipmentService
     */
    public $shipmentService;

    /**
     * @var ShipmentItemService
     */
    public $shipmentItemService;

    /**
     * Plugin Constructor.
     */
    public function __construct()
    {
        $this->defineConstants();
        $this->initHooks();
    }

    private function defineConstants()
    {
        if (!defined('ABSPATH_LENGTH')) {
            define('ABSPATH_LENGTH', strlen(ABSPATH));
        }

        define('WCDPI_ABSPATH',         dirname(WCDPI_PLUGIN_FILE) . '/' );
        define('WCDPI_PLUGIN_BASENAME', plugin_basename(WCDPI_PLUGIN_FILE));
        define('WCDPI_LOG_DIR',         WCDPI_ABSPATH . 'logs/');
        define('WCDPI_TEMPLATES_DIR',   WCDPI_ABSPATH . 'templates/');
        define('WCDPI_MIGRATIONS_DIR',  WCDPI_ABSPATH . 'migrations/');

        // URL's
        define('WCDPI_PLUGIN_URL', plugins_url('woo-dp-internetmarke/'));
        define('WCDPI_ASSETS_URL', WCDPI_PLUGIN_URL . 'assets/');
        define('WCDPI_CSS_URL',    WCDPI_ASSETS_URL . 'css/');
        define('WCDPI_JS_URL',     WCDPI_ASSETS_URL . 'js/');
        define('WCDPI_IMG_URL',    WCDPI_ASSETS_URL . 'images/');

        // Portokasse
        define(
            'WCDPI_WALLET_TOP_UP_URL',
            'https://portokasse.deutschepost.de/portokasse/marketplace/enter-app-payment'
        );
        define(
            'WCDPI_SIGN_UP_URL',
            'https://portokasse.deutschepost.de/portokasse/#!/register/'
        );
        define(
            'WCDPI_KILOTARIF_PDF',
            'https://www.deutschepost.de/content/dam/dpag/images/E_e/Einlieferungslisten/Downloads/dp-el-warenpost-zum-kilotarif-internat-02-2018.pdf'
        );
        define(
            'WCDPI_TRACKING_URL',
            '<a target="_blank" href="https://www.deutschepost.de/sendung/simpleQueryResult.html?form.sendungsnummer=%s&form.einlieferungsdatum_tag=%d&form.einlieferungsdatum_monat=%d&form.einlieferungsdatum_jahr=%d">%s</a>'
        );
    }

    public function init()
    {
        // Credentials Repositories
        $partnerId      = null;
        $keyPhase       = null;
        $marketplaceKey = null;
        $username       = null;
        $password       = null;

        // Credentials Models
        if ($partnerCredentials = Repositories\Credentials\Partner::instance()->get()) {
            $partnerId      = $partnerCredentials->get('OneClickForApplication', 'PartnerId');
            $keyPhase       = $partnerCredentials->get('OneClickForApplication', 'KeyPhase');
            $marketplaceKey = $partnerCredentials->get('OneClickForApplication', 'MarketplaceKey');
            $username       = $partnerCredentials->get('ProdWs', 'Username');
            $password       = $partnerCredentials->get('ProdWs', 'Password');
        }

        // Admin notices
        new Admin\Notice();

        // Connectors (Legacy)
        new FORKLane();

        // Connectors
        try {
            $oneClickForAppConnector    = new Connectors\OneClickForApp\Connector($partnerId, $keyPhase, $marketplaceKey);
            $oneClickForRefundConnector = new Connectors\OneClickForRefund\Connector($partnerId, $keyPhase, $marketplaceKey);
            $packetPlusConnector        = new Connectors\PacketPlus\Connector($partnerId, $keyPhase, $marketplaceKey);
            $prodWSConnector            = new Connectors\ProdWS\Connector($username, $password );
        } catch (SoapFault $sf) {
            $message = '<b>' . __('The plugin could not be loaded, the following error occurred:', 'woo-dp-internetmarke') . '</b>';
            $message .= '<br>' . $sf->getMessage();

            Notice::error($message);
            return;
        }

        // Host Integrations
        new Integrations\Host\ForkLane();
        $oneClickForAppHostIntegration = new Integrations\Host\OneClickForApp($oneClickForAppConnector);
        $oneClickForRefundHostIntegration = new Integrations\Host\OneClickForRefund($oneClickForRefundConnector);
        new Integrations\Host\ProdWS($prodWSConnector);
        $packetPlusHostIntegration = new Integrations\Host\PacketPlus($packetPlusConnector);

        // WooCommerce Integrations
        new Integrations\WooCommerce\ProductData();
        new Integrations\WooCommerce\Orders();

        // Services
        $this->labelService        = new LabelService();
        $this->shipmentService     = new ShipmentService();
        $this->shipmentItemService = new ShipmentItemService();

        // Helper
        $store = new Store();
        new AjaxHandler($store);

        // Components
        new Components\Dashboard($oneClickForAppHostIntegration);
        new Components\Shipments($store, $packetPlusHostIntegration);
        new Components\LabelShop($store);
        new Components\Settings();

        // Admin related
        new Admin\MetaBoxes(
            $oneClickForAppHostIntegration,
            $oneClickForRefundHostIntegration,
            $store
        );

        // Settings
        new Settings\General();
        new Settings\Internetmarke1C4R();
        new Settings\InternetmarkeProdWS();
        new Settings\Logger();
        new Settings\PortChecker();
        new Settings\Tools();
        new Settings\ScheduledTasks();
        new Settings\PacketPlusInternational();
        new Settings\PacketPlusCrossBorder();
        new Settings\Portokasse($oneClickForAppHostIntegration);
        new Settings\Internetmarke1C4A();

        // Export component
        new Export();
    }

    private function initHooks()
    {
        // Init
        add_action('init', array($this, 'init'));

        // Setup
        register_activation_hook(WCDPI_PLUGIN_FILE,   array('\DeutschePost\Internetmarke\Setup', 'activate'));
        register_deactivation_hook(WCDPI_PLUGIN_FILE, array('\DeutschePost\Internetmarke\Setup', 'deactivate'));
        register_uninstall_hook(WCDPI_PLUGIN_FILE,    array('\DeutschePost\Internetmarke\Setup', 'uninstall'));

        // Run DB update
        $this->runUpdate();

        // Misc
        add_action('admin_enqueue_scripts',                        array($this, 'adminEnqueueScripts'));
        add_action('internetmarke_update_product_list',            array($this, 'cronJobUpdateProductList'));
        add_action('internetmarke_update_portokasse_wallet',       array($this, 'cronJobUpdatePortokasseWallet'));
        add_filter('plugin_action_links_' . WCDPI_PLUGIN_BASENAME, array($this, 'pluginActionLinks'));

        add_filter('pre_update_option__wcdpi_settings_general', array($this, 'setDefaultSenderCountry'), 10, 1);
    }

    private function runUpdate()
    {
        // Run update only when
        $runUpdate = true;

        if (key_exists('action', $_REQUEST)) {
            $runUpdateBlacklist = array(
                'delete-plugin',
                'activate',
                'deactivate'
            );

            if (in_array($_REQUEST['action'], $runUpdateBlacklist)) {
                $runUpdate = false;
            }
        }

        if ($runUpdate) {
            Setup::update();
        }
    }

    /**
     * Registers the needed JavaScript and CSS file handlers.
     *
     * @param string $hook
     */
    public function adminEnqueueScripts($hook)
    {
        global $post_type;

        // Select2
        wp_register_style(
            'select2',
            WCDPI_CSS_URL .
            'select2.min.css',
            array(),
            '4.0.13'
        );
        wp_register_script(
            'select2',
            WCDPI_JS_URL . 'select2.full.min.js',
            array(),
            '4.0.13'
        );

        // Vue.js
        wp_register_script('wcdpi-vue.js-dev', 'https://cdn.jsdelivr.net/npm/vue/dist/vue.js');
        wp_register_script('wcdpi-vue.js',     'https://cdn.jsdelivr.net/npm/vue');

        // MD5
        wp_register_script('wcdpi-md5', WCDPI_JS_URL . 'JavaScript-MD5/js/md5.min.js');

        // CSS framework
        wp_register_style('wcdpi-framework', WCDPI_CSS_URL . 'framework.css');

        // Fonts
        wp_register_style('wcdpi-fonts', WCDPI_CSS_URL . 'fonts.css');

        // Vue.js filters
        wp_register_script('wcdpi-vue-filter-shipment-number', WCDPI_JS_URL . 'vue-filter-shipment-number.js');
        wp_register_script('wcdpi-vue-filter-date-time', WCDPI_JS_URL . 'vue-filter-date-time.js');

        // Pages
        wp_register_script(
            'wcdpi-page-settings',
            WCDPI_JS_URL . 'page-settings.js',
            array('wcdpi-md5', 'select2')
        );
        wp_register_style(
            'wcdpi-page-dashboard',
            WCDPI_CSS_URL . 'page-dashboard.css',
            array('wcdpi-framework', 'wcdpi-fonts'),
            WCDPI_VERSION
        );
        wp_register_style(
            'wcdpi-page-shipments',
            WCDPI_CSS_URL . 'page-shipments.css',
            array('wcdpi-framework', 'wcdpi-fonts'),
            WCDPI_VERSION
        );
        wp_register_style(
            'wcdpi-page-label-shop',
            WCDPI_CSS_URL . 'page-label-shop.css',
            array('wcdpi-framework', 'wcdpi-fonts', 'wcdpi-modal-shipment-wizard', 'select2'),
            WCDPI_VERSION
        );
        wp_register_style(
            'wcdpi-page-settings',
            WCDPI_CSS_URL . 'page-settings.css',
            array('wcdpi-framework', 'wcdpi-fonts', 'select2'),
            WCDPI_VERSION
        );

        // Bulk Create
        wp_register_style(
            'wcdpi-page-shipments-bulk-create',
            WCDPI_CSS_URL . 'page-shipments-bulk-create.css',
            array(
                'select2',
                'wcdpi-fonts',
                'wcdpi-framework'
            ),
            WCDPI_VERSION
        );
        wp_register_script(
            'wcdpi-page-shipments-bulk-create',
            WCDPI_JS_URL . 'page-shipments-bulk-create.js',
            array(
                'select2',
                'wcdpi-vue.js-dev'
            ),
            WCDPI_VERSION,
            true
        );

        // Wizard
        wp_register_style(
            'wcdpi-modal-shipment-wizard',
            WCDPI_CSS_URL . 'modal-shipment-wizard.css',
            array(
                'wcdpi-fonts',
                'wcdpi-framework'
            ),
            WCDPI_VERSION
        );
        wp_register_script(
            'wcdpi-modal-shipment-wizard',
            WCDPI_JS_URL . 'modal-shipment-wizard.js',
            array(
                'wcdpi-vue.js-dev'
            ),
            WCDPI_VERSION,
            true
        );

        // Meta boxes
        wp_register_style(
            'wcdpi-meta-box-order-shipments',
            WCDPI_CSS_URL . 'meta-box-order-shipments.css',
            array(
                'wcdpi-fonts',
                'wcdpi-framework'
            ),
            WCDPI_VERSION
        );
        wp_register_style(
            'wcdpi-meta-box-quick-buy',
            WCDPI_CSS_URL . 'meta-box-quick-buy.css',
            array(
                'wcdpi-fonts',
                'wcdpi-framework',
                'select2'
            ),
            WCDPI_VERSION
        );
        wp_register_script(
            'wcdpi-meta-box-order-shipments',
            WCDPI_JS_URL . 'meta-box-order-shipments.js',
            array(
                'wcdpi-vue.js-dev',
                'wcdpi-vue-filter-shipment-number',
                'wcdpi-vue-filter-date-time'
            ),
            WCDPI_VERSION,
            true
        );
        wp_register_script(
            'wcdpi-meta-box-quick-buy',
            WCDPI_JS_URL . 'meta-box-quick-buy.js',
            array(
                'wcdpi-vue.js-dev',
                'select2'
            ),
            WCDPI_VERSION,
            true
        );
        wp_register_script(
            'wcdpi-page-edit-shop_order',
            WCDPI_JS_URL . 'page-edit-shop_order.js',
            array('jquery', 'select2'),
            WCDPI_VERSION,
            true
        );

        // Pages
        wp_register_script(
            'wcdpi-page-dashboard',
            WCDPI_JS_URL . 'page-dashboard.js',
            array('jquery', 'wcdpi-md5'),
            WCDPI_VERSION,
            false
        );
        wp_register_script(
            'wcdpi-page-label-shop',
            WCDPI_JS_URL . 'page-label-shop.js',
            array('jquery', 'wcdpi-vue.js-dev', 'select2'),
            WCDPI_VERSION,
            true
        );

        // Script localization
        wp_localize_script(
            'wcdpi-page-settings',
            'ENV',
            array(
                'nonceUpdateWallet' => wp_create_nonce('wcdpi_update_portokasse_wallet')
            )
        );
        wp_localize_script(
            'wcdpi-page-settings',
            'i18n',
            array(
                'placeholder' => __('Select a shipping product', 'woo-dp-internetmarke')
            )
        );
        wp_localize_script(
            'wcdpi-page-edit-shop_order',
            'i18n',
            array(
                'filterPlaceholder' => __('Filter by shipping region', 'woo-dp-internetmarke')
            )
        );
        wp_localize_script(
            'wcdpi-page-dashboard',
            'security',
            array(
                'walletUpdate' => wp_create_nonce('wcdpi_update_portokasse_wallet')
            )
        );

        // Dashboard
        if ($hook === 'toplevel_page_internetmarke') {
            wp_enqueue_style('wcdpi-page-dashboard');
            wp_enqueue_script('wcdpi-page-dashboard');
        }

        // Shipments
        if ($hook === 'internetmarke_page_internetmarke_shipments') {
            wp_enqueue_style('wcdpi-page-shipments');
            wp_enqueue_script('wcdpi-vue.js-dev');
        }

        if ($hook === 'admin_page_internetmarke_shipments_bulk_create') {
            wp_localize_script(
                'wcdpi-page-shipments-bulk-create',
                'i18n',
                array(
                    'placeholder' => __('Select a shipping product', 'woo-dp-internetmarke')
                )
            );

            wp_enqueue_style('wcdpi-page-shipments-bulk-create');
            wp_enqueue_script('wcdpi-page-shipments-bulk-create');
        }

        // Label shop
        if ($hook === 'internetmarke_page_internetmarke_label_shop') {
            wp_localize_script(
                'wcdpi-page-label-shop',
                'security',
                array(
                    'userSearch' => wp_create_nonce('wcdpi_dropdown_user_search'),
                    'productSearch' => wp_create_nonce('search-products'),
                    'addLineItem' => wp_create_nonce('wcdpi_label_shop_add_line_item')
                )
            );
            wp_localize_script(
                'wcdpi-page-label-shop',
                'i18n',
                array(
                    'placeholder' => array(
                        'userSearch' => __('Search for customers...', 'woo-dp-internetmarke'),
                        'productSearch' => __('Search for products...', 'woo-dp-internetmarke')
                    )
                )
            );

            wp_enqueue_style('wcdpi-page-label-shop');
            wp_enqueue_script('wcdpi-page-label-shop');
        }

        // Settings
        if ($hook === 'internetmarke_page_internetmarke_settings' ||
            $hook === 'toplevel_page_internetmarke_settings'
        ) {
            wp_enqueue_style('wcdpi-page-settings');
            wp_enqueue_script('wcdpi-page-settings');
        }

        // WooCommerce Order Overview
        if ($hook === 'edit.php' && $post_type === 'shop_order') {
            wp_enqueue_script('wcdpi-page-edit-shop_order');
        }

        // Edit order
        if ($hook === 'post.php' && array_key_exists('post', $_GET) && wc_get_order($_GET['post'])) {
            wp_enqueue_style('wcdpi-meta-box-order-shipments');
            wp_enqueue_style('wcdpi-meta-box-quick-buy');
            wp_enqueue_style('wcdpi-modal-shipment-wizard');
            wp_enqueue_script('wcdpi-meta-box-order-shipments');
            wp_enqueue_script('wcdpi-meta-box-quick-buy');

            wp_localize_script(
                'wcdpi-meta-box-quick-buy',
                'i18n',
                array(
                    'placeholder' => __('Select a shipping product', 'woo-dp-internetmarke')
                )
            );
        }
    }

    /**
     * Fires when the scheduled task / cron job is scheduled to update the
     * product list.
     */
    public function cronJobUpdateProductList()
    {
        do_action('wcdpi_integration_prodws_update_product_list');
        do_action('wcdpi_integration_oneclickforapp_update_contract_products');
        Internetmarke()->logger()->info('SCHEDULED_TASK: Product list updated');
    }

    /**
     * Fires when the scheduled task / cron job is scheduled to update the
     * portokasse wallet.
     */
    public function cronJobUpdatePortokasseWallet()
    {
        do_action('wcdpi_integration_oneclickforapp_update_wallet');
        Internetmarke()->logger()->info('SCHEDULED_TASK: Portokasse wallet updated');
    }

    /**
     * @param array $links
     *
     * @return array
     */
    public function pluginActionLinks($links)
    {
        $link = sprintf(
            '<a href="%s">%s</a>',
            admin_url('admin.php?page=internetmarke_settings'),
            __('Settings', 'woo-dp-internetmarke')
        );

        $actionLinks = array('settings' => $link);

        return array_merge($actionLinks, $links);
    }

    public function setDefaultSenderCountry($value)
    {
        $optionsCrossBorder = get_option('_wcdpi_settings_packet_plus_cross_border', array());

        if ($optionsCrossBorder) {
            $activationCrossBorder = (
                key_exists('_wcdpi_packet_plus_cross_border_activation', $optionsCrossBorder) &&
                $optionsCrossBorder['_wcdpi_packet_plus_cross_border_activation'] == 1)
            ;
        } else {
            $activationCrossBorder = false;
        }

        if (!$activationCrossBorder) {
            $value['_wcdpi_default_sender_country'] = 'DE';
        }

        return $value;
    }

    /**
     * @param string $file "internetmarke.log", "internetmarke_api.log", or "internetmarke_shipments.log"
     *
     * @return Logger
     */
    public function logger($file = Logger::LOG_INTERNETMARKE)
    {
        if (self::$logger === null) {
            self::$logger = new Logger();
        }

        self::$logger->setLogger($file);

        return self::$logger;
    }
}