<?php

namespace DeutschePost\Internetmarke;

use Monolog\Handler\StreamHandler;
use Monolog\Logger as MonologLogger;
use Psr\Log\LoggerInterface;
use DeutschePost\Internetmarke\Abstracts\Singleton;

defined('ABSPATH') || exit;

class Logger extends Singleton implements LoggerInterface
{
    /**
     * @var string
     */
    const LOG_INTERNETMARKE = 'internetmarke.log';

    /**
     * @var string
     */
    const LOG_INTERNETMARKE_API = 'internetmarke_api.log';

    /**
     * @var string
     */
    const LOG_INTERNETMARKE_SHIPMENTS = 'internetmarke_shipments.log';

    /**
     * @var MonologLogger
     */
    protected $logger;

    /**
     * @var MonologLogger
     */
    protected $internetmarkeLogger;

    /**
     * @var MonologLogger
     */
    protected $apiLogger;

    /**
     * @var MonologLogger
     */
    protected $shipmentsLogger;

    /**
     * Logger constructor.
     */
    public function __construct()
    {
        // Set up the logger instances
        $this->internetmarkeLogger = new MonologLogger('INTERNETMARKE');
        $this->apiLogger           = new MonologLogger('API');
        $this->shipmentsLogger     = new MonologLogger('SHIPMENTS');

        // Push the needed handlers
        try {
            $this->internetmarkeLogger->pushHandler(
                new StreamHandler(
                    WCDPI_LOG_DIR . self::LOG_INTERNETMARKE,
                    MonologLogger::DEBUG
                )
            );
            $this->apiLogger->pushHandler(
                new StreamHandler(
                    WCDPI_LOG_DIR . self::LOG_INTERNETMARKE_API,
                    MonologLogger::NOTICE
                )
            );

            $this->shipmentsLogger->pushHandler(
                new StreamHandler(
                    WCDPI_LOG_DIR . self::LOG_INTERNETMARKE_SHIPMENTS,
                    MonologLogger::NOTICE
                )
            );
        } catch (\Exception $e) {
        }
    }

    /**
     * Sets the logger that needs to be used.
     *
     * @param $file
     */
    public function setLogger($file)
    {
        switch ($file) {
            case self::LOG_INTERNETMARKE_API:
                $this->logger = $this->apiLogger;
                break;
            case self::LOG_INTERNETMARKE_SHIPMENTS:
                $this->logger = $this->shipmentsLogger;
                break;
            default:
                $this->logger = $this->internetmarkeLogger;
                break;
        }
    }

    /**
     * System is unusable.
     *
     * @param string $message The log message
     * @param array  $context The log context
     *
     * @return bool Whether the record has been processed
     */
    public function emergency($message, array $context = array())
    {
        return $this->logger->addRecord(MonologLogger::EMERGENCY, $message, $context);
    }

    /**
     * Action must be taken immediately.
     *
     * Example: Entire website down, database unavailable, etc. This should
     * trigger the SMS alerts and wake you up.
     *
     * @param string $message The log message
     * @param array  $context The log context
     *
     * @return bool Whether the record has been processed
     */
    public function alert($message, array $context = array())
    {
        return $this->logger->addRecord(MonologLogger::ALERT, $message, $context);
    }

    /**
     * Critical conditions.
     *
     * Example: Application component unavailable, unexpected exception.
     *
     * @param string $message The log message
     * @param array  $context The log context
     *
     * @return bool Whether the record has been processed
     */
    public function critical($message, array $context = array())
    {
        return $this->logger->addRecord(MonologLogger::CRITICAL, $message, $context);
    }

    /**
     * Runtime errors that do not require immediate action but should typically
     * be logged and monitored.
     *
     * @param string $message The log message
     * @param array  $context The log context
     *
     * @return bool Whether the record has been processed
     */
    public function error($message, array $context = array())
    {
        return $this->logger->addRecord(MonologLogger::ERROR, $message, $context);
    }

    /**
     * Exceptional occurrences that are not errors.
     *
     * Example: Use of deprecated APIs, poor use of an API, undesirable things
     * that are not necessarily wrong.
     *
     * @param string $message The log message
     * @param array  $context The log context
     *
     * @return bool Whether the record has been processed
     */
    public function warning($message, array $context = array())
    {
        return $this->logger->addRecord(MonologLogger::WARNING, $message, $context);
    }

    /**
     * Normal but significant events.
     *
     * @param string $message The log message
     * @param array  $context The log context
     *
     * @return bool Whether the record has been processed
     */
    public function notice($message, array $context = array())
    {
        return $this->logger->addRecord(MonologLogger::NOTICE, $message, $context);
    }

    /**
     * Interesting events.
     *
     * Example: User logs in, SQL logs.
     *
     * @param string $message The log message
     * @param array  $context The log context
     *
     * @return bool Whether the record has been processed
     */
    public function info($message, array $context = array())
    {
        return $this->logger->addRecord(MonologLogger::INFO, $message, $context);
    }

    /**
     * Detailed debug information.
     *
     * @param string $message The log message
     * @param array  $context The log context
     *
     * @return bool Whether the record has been processed
     */
    public function debug($message, array $context = array())
    {
        return $this->logger->addRecord(MonologLogger::DEBUG, $message, $context);
    }

    /**
     * Logs with an arbitrary level.
     *
     * @param int    $level   The log level
     * @param string $message The log message
     * @param array  $context The log context
     *
     * @return bool Whether the record has been processed
     */
    public function log($level, $message, array $context = array())
    {
        $level = MonologLogger::toMonologLevel($level);

        return $this->logger->addRecord($level, $message, $context);
    }
}