<?php

namespace DeutschePost\Internetmarke\Services;

use Exception;
use SoapFault;
use DeutschePost\Internetmarke\Enums\ShippingRegion;
use DeutschePost\Internetmarke\Models\Resources\ProductSales as ProductSalesResourceModel;
use DeutschePost\Internetmarke\Repositories\Resources\ProductSales as ProductSalesResourceRepository;
use WC_Order;
use WC_Order_Item_Product;
use WC_Product;
use function DeutschePost\Internetmarke\Internetmarke;

defined('ABSPATH') || exit;

class LabelService
{
    /**
     * Label constructor.
     */
    public function __construct()
    {
        add_action('wp_ajax_wcdpi_preview_label', array($this, 'previewLabel'));
    }

    /**
     * Checks if all needed data is present for the order's line items, so that a harmonized label can be generated.
     *
     * @param int|WC_Order $order    WooCommerce Order or Order ID
     * @param int          $prodwsId Product Information Webservice ID
     *
     * @return bool
     */
    public function isHarmonizedLabelAvailable($order, $prodwsId)
    {
        if (is_numeric($order)) {
            $order = wc_get_order($order);
        }

        if (!$order || !$order instanceof WC_Order) {
            return false;
        }

        // Not available for incompatible labels
        if (!in_array($prodwsId, wcdpi_get_harmonized_labels())) {
            return false;
        }

        // Not available for domestic shipments
        if (strtoupper($order->get_shipping_country()) === 'DE') {
            return false;
        }

        // Unavailable for EU if an incompatible label was selected
        if (in_array(strtoupper($order->get_shipping_country()), wcdpi_get_eu_countries())
            && !in_array($prodwsId, wcdpi_get_harmonized_labels(ShippingRegion::EUROPEAN_UNION))
        ) {
            return false;
        }

        // Unavailable for ROW...
        if (!in_array(strtoupper($order->get_shipping_country()), wcdpi_get_eu_countries())) {
            // ... if an incompatible label was selected
            if (!in_array($prodwsId, wcdpi_get_harmonized_labels(ShippingRegion::REST_OF_WORLD))) {
                return false;
            }

            // ... if the product data is missing
            /** @var WC_Order_Item_Product $orderItemProduct */
            foreach ($order->get_items() as $orderItemProduct) {
                /** @var WC_Product $product */
                $product       = $orderItemProduct->get_product();
                $productId     = $orderItemProduct->get_product_id();
                $productWeight = $product->get_weight();

                $customsDeclarationText   = get_post_meta($productId, 'wcdpi_customs_declaration_text',    true);
                $customsDeclarationPrice  = get_post_meta($productId, 'wcdpi_customs_declaration_price',   true);
                $customsDeclarationHsCode = get_post_meta($productId, 'wcdpi_customs_declaration_hs_code', true);
                $customsDeclarationOrigin = get_post_meta($productId, 'wcdpi_customs_declaration_origin',  true);

                if (!$customsDeclarationText || !$customsDeclarationHsCode || !$customsDeclarationOrigin || !$productWeight) {
                    return false;
                }
            }
        }

        return true;
    }

    /**
     * AJAX Handler for the label preview action.
     */
    public function previewLabel()
    {
        // Validate request.
        if ($_SERVER['REQUEST_METHOD'] !== 'POST') {
            wp_send_json('Invalid request.', 500);
        }

        if (!current_user_can('preview_label')) {
            wp_send_json('Invalid request.', 500);
        }

        if (!array_key_exists('prodwsId', $_POST)) {
            wp_send_json('ProdWS ID is missing.', 500);
        }

        $prodwsId = sanitize_text_field($_POST['prodwsId']);

        /** @var ProductSalesResourceModel $productSales */
        if (!$productSales = ProductSalesResourceRepository::instance()->findBy(array('prodws_id' => $prodwsId))) {
            wp_send_json('The selected Product could not be found.', 500);
        }

        $harmonizedLabel = ($_POST['harmonizedLabel'] == 'true');
        $amount          = intval($_POST['amount']);
        $infoText        = $productSales->getInformationText();
        $price           = $productSales->getPrice() / 100;
        $total           = wcdpi_format_price($amount * $price);
        $dimensions      = wcdpi_generate_dimensions_html($productSales);

        if ($harmonizedLabel) {
            $url = apply_filters('wcdpi_integration_packetplus_preview_label', $prodwsId);
        } else {
            try {
                $url = apply_filters(
                    'wcdpi_integration_oneclickforapp_preview_label',
                    $productSales->getProductCode(),
                    'AddressZone'
                );
            } catch (SoapFault $sf) {
                Internetmarke()->logger()->error($sf->getMessage(), $sf->getTrace());
                wp_send_json($sf->getMessage(), 500);
            } catch (Exception $e) {
                Internetmarke()->logger()->error($e->getMessage(), $e->getTrace());
                wp_send_json($e->getMessage(), 500);
            }
        }

        wp_send_json(
            array(
                'url'         => $url,
                'total'       => $total,
                'info'        => $infoText,
                'dimensions'  => $dimensions
            )
        );
    }
}