<?php

namespace DeutschePost\Internetmarke\Services;

use DeutschePost\Internetmarke\Address;
use DeutschePost\Internetmarke\Connectors\OneClickForApp\Response\CheckoutShoppingCartPDF as CheckoutShoppingCartPDFResponse;
use DeutschePost\Internetmarke\Connectors\OneClickForRefund\Response\RetoureVouchers;
use DeutschePost\Internetmarke\Connectors\PacketPlus\Response\CreateOrder as CreateOrderResponse;
use DeutschePost\Internetmarke\Enums\ShipmentAttachment;
use DeutschePost\Internetmarke\Enums\ShipmentStatus;
use DeutschePost\Internetmarke\Logger;
use DeutschePost\Internetmarke\Models\Resources\ProductSales as ResourceProductSalesModel;
use DeutschePost\Internetmarke\Models\Resources\Shipment as ResourceShipmentModel;
use DeutschePost\Internetmarke\Repositories\Resources\ProductSales as ProductSalesResourceRepository;
use DeutschePost\Internetmarke\Repositories\Resources\Shipment as ShipmentResourceRepository;
use DeutschePost\Internetmarke\Repositories\Resources\ShipmentAttachment as ShipmentAttachmentResourceRepository;
use DeutschePost\Internetmarke\Repositories\Resources\ShipmentItem as ShipmentItemResourceRepository;
use function DeutschePost\Internetmarke\Internetmarke;

defined('ABSPATH') || exit;

class ShipmentService
{
    /**
     * Creates a new shipment with the "PREPARED" status.
     *
     * @param int     $orderId         WooCommerce Order ID
     * @param string  $prodwsId        ProdWS ID
     * @param bool    $harmonizedLabel Harmonized Label Identifier
     * @param Address $sender          Sender address object
     * @param Address $recipient       Recipient address object
     * @param array   $lineItems       Line item data
     *
     * @return ResourceShipmentModel
     */
    public function prepare($orderId, $prodwsId, $harmonizedLabel, $sender, $recipient, $lineItems)
    {
        $shipmentNatureType   = 'SALE_GOODS';
        $shipmentServiceLevel = 'PRIORITY';
        $ekp                  = '0000010000';
        $settingsPacketPlus   = get_option('_wcdpi_settings_packet_plus_international', array());

        /** @var ResourceProductSalesModel $productSales */
        $productSales = ProductSalesResourceRepository::instance()->findBy(
            array(
                'prodws_id' => $prodwsId
            )
        );

        if (array_key_exists('shipment_nature_type', $_POST)) {
            $shipmentNatureType = $_POST['shipment_nature_type'];
        }

        if (array_key_exists('service_level', $_POST)) {
            $shipmentServiceLevel = $_POST['service_level'];
        }

        if (array_key_exists('_wcdpi_packet_plus_international_ekp', $settingsPacketPlus)) {
            $ekp = $settingsPacketPlus['_wcdpi_packet_plus_international_ekp'];
        }

        $shipmentData = array(
            'order_id'                 => $orderId,
            'prodws_id'                => $prodwsId,
            'harmonized_label'         => ($harmonizedLabel) ? 1 : 0,
            'ekp'                      => $ekp,
            'price'                    => $productSales->getPrice(),
            'nature_type'              => $shipmentNatureType,
            'service_level'            => $shipmentServiceLevel,
            'status'                   => ShipmentStatus::PREPARED,
            'sender_first_name'        => $sender->getFirstName(),
            'sender_last_name'         => $sender->getLastName(),
            'sender_company_name'      => $sender->getCompany(),
            'sender_email'             => $sender->getEmail(),
            'sender_phone'             => $sender->getPhone(),
            'sender_contact_name'      => $sender->getContactName(),
            'sender_street'            => $sender->getStreet(),
            'sender_house_number'      => $sender->getHouseNumber(),
            'sender_additional'        => $sender->getAdditional(),
            'sender_zip'               => $sender->getZip(),
            'sender_city'              => $sender->getCity(),
            'sender_country_alpha2'    => $sender->getCountry(),
            'sender_country_alpha3'    => $sender->getCountryIso3(),
            'recipient_first_name'     => $recipient->getFirstName(),
            'recipient_last_name'      => $recipient->getLastName(),
            'recipient_company_name'   => $recipient->getCompany(),
            'recipient_email'          => $recipient->getEmail(),
            'recipient_phone'          => $recipient->getPhone(),
            'recipient_street'         => $recipient->getStreet(),
            'recipient_house_number'   => $recipient->getHouseNumber(),
            'recipient_additional'     => $recipient->getAdditional(),
            'recipient_zip'            => $recipient->getZip(),
            'recipient_city'           => $recipient->getCity(),
            'recipient_state'          => $recipient->getState(),
            'recipient_country_alpha2' => $recipient->getCountry(),
            'recipient_country_alpha3' => $recipient->getCountryIso3()
        );

        /** @var ResourceShipmentModel $shipment */
        $shipment = ShipmentResourceRepository::instance()->insert($shipmentData);

        foreach ($lineItems as $lineItem) {
            if (!$lineItem['qtySelected'] || $lineItem['qtySelected'] == 0) {
                continue;
            }

            $shipmentItemData = array(
                'shipment_id'                 => $shipment->getId(),
                'order_id'                    => $orderId,
                'product_id'                  => $lineItem['productId'],
                'product_quantity'            => $lineItem['qtySelected'],
                'product_price'               => (float)$lineItem['price'],
                'product_weight'              => (int)$lineItem['weight'],
                'product_hs_code'             => $lineItem['hsCode'],
                'product_customs_declaration' => $lineItem['customsDeclaration'],
                'product_origin'              => $lineItem['countryOfManufacture']
            );

            ShipmentItemResourceRepository::instance()->insert($shipmentItemData);
        }

        Internetmarke()->logger(Logger::LOG_INTERNETMARKE_SHIPMENTS)->notice('1 Shipment prepared successfully.');

        return $shipment;
    }

    /**
     * Orders the shipping label for a shipment.
     *
     * @param int $shipmentId
     *
     * @return ResourceShipmentModel
     */
    public function order($shipmentId)
    {
        /** @var ResourceShipmentModel $shipment */
        $shipment = ShipmentResourceRepository::instance()->find($shipmentId);

        if ($shipment->isHarmonizedLabel()) {
            /** @var CreateOrderResponse $purchaseShippingLabelResponse */
            $purchaseShippingLabelResponse = apply_filters('wcdpi_integration_packetplus_purchase_label', $shipment);
            $shipments = $purchaseShippingLabelResponse->getShipments();

            $shipmentData = array(
                'shop_pp_order_id' => $purchaseShippingLabelResponse->getOrderId(),
                'awb_id'           => $shipments[0]->awb,
                'pp_shipment_id'   => $shipments[0]->items[0]->id,
                'tracking'         => $shipments[0]->items[0]->barcode,
                'voucher_id'       => $shipments[0]->items[0]->voucherId,
                'status'           => ShipmentStatus::READY_TO_SHIP,
                'purchase_dt'      => gmdate('Y-m-d H:i:s'),
                'meta_update_dt'   => gmdate('Y-m-d H:i:s'),
                'meta_update_by'   => get_current_user_id()
            );
            $shipmentAttachmentData = array(
                'shipment_id'    => $shipmentId,
                'url'            => $this->getHarmonizedShippingLabelUrl($shipments[0]->items[0]->id),
                'type'           => ShipmentAttachment::LABEL,
                'meta_insert_dt' => gmdate('Y-m-d H:i:s'),
                'meta_insert_by' => get_current_user_id()
            );
        } else {
            /** @var CheckoutShoppingCartPDFResponse $purchaseShippingLabelResponse */
            $purchaseShippingLabelResponse = apply_filters('wcdpi_integration_oneclickforapp_purchase_label', $shipment);
            $shoppingCart = $purchaseShippingLabelResponse->getShoppingCart();

            $shipmentData = array(
                'shop_order_id'  => $shoppingCart->shopOrderId,
                'voucher_id'     => $shoppingCart->voucherList->voucher->voucherId,
                'tracking'       => $this->getShipmentTrackingNumber($shoppingCart, $shipment),
                'status'         => ShipmentStatus::READY_TO_SHIP,
                'purchase_dt'    => gmdate('Y-m-d H:i:s'),
                'meta_update_dt' => gmdate('Y-m-d H:i:s'),
                'meta_update_by' => get_current_user_id()
            );
            $shipmentAttachmentData = array(
                'shipment_id'    => $shipmentId,
                'url'            => $purchaseShippingLabelResponse->getLink(),
                'type'           => ShipmentAttachment::LABEL,
                'meta_insert_dt' => gmdate('Y-m-d H:i:s'),
                'meta_insert_by' => get_current_user_id()
            );
        }

        ShipmentResourceRepository::instance()->update($shipmentId, $shipmentData);
        ShipmentAttachmentResourceRepository::instance()->insert($shipmentAttachmentData);

        Internetmarke()->logger(Logger::LOG_INTERNETMARKE_SHIPMENTS)->notice('1 Shipment created successfully.');

        $shipment = ShipmentResourceRepository::instance()->find($shipmentId);

        wcdpi_add_shipment_success_order_note($shipment);

        return $shipment;
    }

    /**
     * Cancels the shipment and refunds the shipping label (if possible).
     *
     * @param int $shipmentId
     *
     * @return bool|RetoureVouchers
     */
    public function cancel($shipmentId)
    {
        /** @var ResourceShipmentModel $shipment */
        $shipment = ShipmentResourceRepository::instance()->find($shipmentId);

        // Refunding a label is not possible for harmonized labels
        if ($shipment->isHarmonizedLabel()) {
            return false;
        }

        /** @var RetoureVouchers $refundObject */
        $refundObject = apply_filters('wcdpi_integration_oneclickforrefund_refund_label', $shipment);

        if (!$refundObject) {
            return false;
        }

        if ($refundObject->getRetoureTransactionId()) {
            $shipmentData = array(
                'retoure_transaction_id' => $refundObject->getRetoureTransactionId(),
                'status' => ShipmentStatus::REFUNDED,
                'meta_update_dt' => gmdate('Y-m-d H:i:s'),
                'meta_update_by' => get_current_user_id()
            );

            ShipmentResourceRepository::instance()->update($shipmentId, $shipmentData);
        }

        Internetmarke()->logger(Logger::LOG_INTERNETMARKE_SHIPMENTS)->notice('1 Shipment canceled successfully.');

        return $refundObject;
    }

    /**
     * Permanently deletes a shipment including possible attachments and items.
     *
     * @param $shipmentIds
     *
     * @return bool|int
     */
    public function delete($shipmentIds)
    {
        ShipmentAttachmentResourceRepository::instance()->deleteBy(array('shipment_id' => $shipmentIds));
        ShipmentItemResourceRepository::instance()->deleteBy(array('shipment_id' => $shipmentIds));

        Internetmarke()->logger(Logger::LOG_INTERNETMARKE_SHIPMENTS)->notice('1 Shipment deleted successfully.');

        return ShipmentResourceRepository::instance()->delete($shipmentIds);
    }

    /**
     * Changes the status of an existing shipment.
     *
     * @param int $shipmentId
     * @param int $statusId
     * @return bool
     */
    public function setStatus($shipmentId, $statusId)
    {
        /** @var false|ResourceShipmentModel $shipment */
        $shipment = ShipmentResourceRepository::instance()->find($shipmentId);

        if (!$shipment) {
            return false;
        }

        $allowed = array(
            ShipmentStatus::PREPARED,
            ShipmentStatus::READY_TO_SHIP,
            ShipmentStatus::SHIPPED,
            ShipmentStatus::DELIVERED,
            ShipmentStatus::REFUNDED,
            ShipmentStatus::RETURNED,
            ShipmentStatus::DELETED,
            ShipmentStatus::FAILED
        );

        if (!in_array($statusId, $allowed)) {
            return false;
        }

        $update = ShipmentResourceRepository::instance()->update(
            $shipmentId,
            array(
                'status' => $statusId
            )
        );

        if (!$update) {
            return false;
        }

        return true;
    }

    /**
     * Retrieves the URL from which the harmonized label can be downloaded from.
     *
     * @param int $id
     *
     * @return string
     */
    private function getHarmonizedShippingLabelUrl($id)
    {
        return admin_url(sprintf('admin-post.php?action=wcdpi_download_packet_plus_label&item=%d', $id));
    }

    /**
     * Retrieves the tracking number from a non-harmonized shipment
     *
     * @param \stdClass             $shoppingCart
     * @param ResourceShipmentModel $shipment
     *
     * @return string|null
     */
    private function getShipmentTrackingNumber($shoppingCart, $shipment)
    {
        $trackingNumber = null;

        // Einschreiben
        if (property_exists($shoppingCart->voucherList->voucher, 'trackId')) {
            $trackingNumber = $shoppingCart->voucherList->voucher->trackId;
        }

        // PRIO
        elseif ($shipment->getProdwsId() >= 12756
            && $shipment->getProdwsId() <= 12761
            && property_exists($shoppingCart->voucherList->voucher, 'voucherId')
        ) {
            $trackingNumber = $shoppingCart->voucherList->voucher->voucherId;
        }

        return $trackingNumber;
    }
}