<?php

namespace DeutschePost\Internetmarke\Settings;

use DeutschePost\Internetmarke\Abstracts\Settings as AbstractSettings;
use function DeutschePost\Internetmarke\Internetmarke;

defined('ABSPATH') || exit;

class PacketPlusCrossBorder extends AbstractSettings
{
    /**
     * @var array
     */
    protected $settings;

    /**
     * PacketPlusCrossBorder constructor.
     */
    public function __construct()
    {
        $this->settings = get_option('_wcdpi_settings_packet_plus_cross_border', array());

        add_action('admin_init', array($this, 'init'));
    }

    /**
     * Registers the "PacketPlusCrossBorder" setting group.
     *
     * @see https://developer.wordpress.org/reference/functions/register_setting/#parameters
     */
    public function init()
    {
        $loggedIn = array_key_exists('_wcdpi_packet_plus_cross_border_terms_and_conditions', $this->settings);

        register_setting(
            '_wcdpi_settings_group_packet_plus_cross_border',
            '_wcdpi_settings_packet_plus_cross_border',
            array(
                'sanitize_callback' => array($this, 'sanitizeSettings')
            )
        );

        add_settings_section(
            '_wcdpi_packet_plus_cross_border_status_section',
            __('Service status', 'woo-dp-internetmarke'),
            null,
            '_wcdpi_packet_plus_cross_border'
        );

        add_settings_field(
            '_wcdpi_packet_plus_cross_border_activation',
            __('Enabled', 'woo-dp-internetmarke'),
            array($this, 'packetPlusCrossBorderActivation'),
            '_wcdpi_packet_plus_cross_border',
            '_wcdpi_packet_plus_cross_border_status_section'
        );

        add_settings_section(
            '_wcdpi_packet_plus_cross_border_section',
            __('Account', 'woo-dp-internetmarke'),
            array($this, 'packetPlusCrossBorderHeader'),
            '_wcdpi_packet_plus_cross_border'
        );

        add_settings_field(
            '_wcdpi_packet_plus_cross_border_terms_and_conditions',
            __('Terms and conditions', 'woo-dp-internetmarke'),
            array($this, 'packetPlusCrossBorderTermsAndConditions'),
            '_wcdpi_packet_plus_cross_border',
            '_wcdpi_packet_plus_cross_border_section'
        );

        if ($loggedIn) {
            add_settings_field(
                '_wcdpi_packet_plus_cross_border_logout',
                __('Reset', 'woo-dp-internetmarke'),
                array($this, 'packetPlusCrossBorderLogout'),
                '_wcdpi_packet_plus_cross_border',
                '_wcdpi_packet_plus_cross_border_section'
            );
        }
    }

    public function sanitizeSettings($settings)
    {
        if (array_key_exists('_wcdpi_packet_plus_cross_border_activation', $settings)
            && $settings['_wcdpi_packet_plus_cross_border_activation'] == 1
        ) {
            if (!$settings['_wcdpi_packet_plus_cross_border_terms_and_conditions']) {
                add_settings_error(
                    '_wcdpi_settings_group_packet_plus_cross_border',
                    'login_error',
                    __('You must accept terms and conditions to use this service.', 'woo-dp-internetmarke'),
                    'error'
                );

                return $this->settings;
            }
        }

        elseif (key_exists('_wcdpi_packet_plus_cross_border_activation', $settings)
            && $settings['_wcdpi_packet_plus_cross_border_activation'] == 0
        ) {
            // TODO Force default sender to GERMANY
        }

        if (array_key_exists('packet_plus_cross_border_logout', (array)$_POST)) {
            try {
                do_action('_wcdpi_packet_plus_cross_border_logout');
                return array();
            } catch (\Exception $e) {
                Internetmarke()->logger()->error($e->getMessage(), $e->getTrace());
            }
        }

        return $settings;
    }

    public function packetPlusCrossBorderHeader()
    {
        _e("Here you can log in with your Packet Plus credentials. After activating the service, the products of Packet Plus are available.", 'woo-dp-internetmarke');
    }

    public function packetPlusCrossBorderActivation()
    {
        $field = '_wcdpi_packet_plus_cross_border_activation';
        $value = $this->getSettingValue($field, true);

        $html = sprintf('<select id="%s" name="_wcdpi_settings_packet_plus_cross_border[%s]">', $field, $field);

        $html .= sprintf(
            '<option value="0" %s>%s</option>',
            ($value == 0) ? 'selected="selected"' : '',
            __('No', 'woo-dp-internetmarke')
        );

        $html .= sprintf(
            '<option value="1" %s>%s</option>',
            ($value == 1) ? 'selected="selected"' : '',
            __('Yes', 'woo-dp-internetmarke')
        );

        $html .= '</select>';
        $html .= $this->createDescription(
            __('When you are logged in, the service is automatically activated.', 'woo-dp-internetmarke')
        );

        echo $html;
    }

    public function packetPlusCrossBorderTermsAndConditions()
    {
        $field = '_wcdpi_packet_plus_cross_border_terms_and_conditions';
        $value = $this->getSettingValue($field, false);

        $html = '<fieldset>';
        $html .= sprintf('<label for="%s">', $field);
        $html .= sprintf(
            '<input id="%s" type="checkbox" name="_wcdpi_settings_packet_plus_cross_border[%s]" value="1" %s/>',
            $field,
            $field,
            checked(true, $value, false)
        );
        $html .= sprintf(
            __('Terms and conditions of the <b>Deutsche Post</b> (incl. <b>Packet Plus</b>) can be found <a href="%s" target="_blank">here</a>', 'woo-dp-internetmarke'),
            'https://www.deutschepost.de/'
        );
        $html .= '</label>';

        $html .= $this->createDescription(
            __('By ticking the boxes and clicking the button I confirm that I have read and consent to the terms listed above.', 'woo-dp-internetmarke')
        );

        $html .= '</fieldset>';

        echo $html;
    }

    public function packetPlusCrossBorderLogout()
    {
        $html = '';
        $html .= sprintf(
            '<button type="submit" name="packet_plus_cross_border_logout">%s</button>',
            __('Logout', 'woo-dp-internetmarke')
        );
        $html .= $this->createDescription(__('A logout resets all settings on this page.', 'woo-dp-internetmarke'));

        echo $html;
    }
}