<?php

namespace DeutschePost\Internetmarke\Settings;

use Exception;
use DeutschePost\Internetmarke\Abstracts\Settings as AbstractSettings;
use DeutschePost\Internetmarke\Interfaces\CredentialsRepository as CredentialsRepositoryInterface;
use DeutschePost\Internetmarke\Repositories\Credentials\User as UserCredentialsRepository;
use function DeutschePost\Internetmarke\Internetmarke;

defined('ABSPATH') || exit;

class PacketPlusInternational extends AbstractSettings
{
    /**
     * @var string
     */
    const DEFAULT_EKP = '0000010000';

    /**
     * @var array
     */
    protected $settings;

    /**
     * PacketPlusInternational constructor.
     */
    public function __construct()
    {
        $this->settings = get_option('_wcdpi_settings_packet_plus_international', array());

        add_action('admin_init', array($this, 'init'));
    }

    /**
     * Registers the "PacketPlusInternational" setting group.
     *
     * @see https://developer.wordpress.org/reference/functions/register_setting/#parameters
     */
    public function init()
    {
        register_setting(
            '_wcdpi_settings_group_packet_plus_international',
            '_wcdpi_settings_packet_plus_international',
            array(
                'sanitize_callback' => array($this, 'sanitizeSettings')
            )
        );

        add_settings_section(
            '_wcdpi_packet_plus_international_status_section',
            __('Service status', 'woo-dp-internetmarke'),
            null,
            '_wcdpi_packet_plus_international'
        );

        add_settings_section(
            '_wcdpi_packet_plus_international_section',
            __('Account', 'woo-dp-internetmarke'),
            array($this, 'packetPlusInternationalHeader'),
            '_wcdpi_packet_plus_international'
        );

        add_settings_field(
            '_wcdpi_packet_plus_international_status',
            __('Enabled', 'woo-dp-internetmarke'),
            array($this, 'packetPlusInternationalActivation'),
            '_wcdpi_packet_plus_international',
            '_wcdpi_packet_plus_international_status_section'
        );

        add_settings_field(
            '_wcdpi_packet_plus_international_ekp',
            __('EKP', 'woo-dp-internetmarke'),
            array($this, 'packetPlusInternationalEkp'),
            '_wcdpi_packet_plus_international',
            '_wcdpi_packet_plus_international_section'
        );

        add_settings_field(
            '_wcdpi_packet_plus_international_default_shipment_nature_type',
            __('Default shipment nature type', 'woo-dp-internetmarke'),
            array($this, 'packetPlusInternationalDefaultShipmentNatureType'),
            '_wcdpi_packet_plus_international',
            '_wcdpi_packet_plus_international_section'
        );

        add_settings_field(
            '_wcdpi_packet_plus_international_default_line_item_origin',
            __('Default line item origin', 'woo-dp-internetmarke'),
            array($this, 'packetPlusInternationalDefaultLineItemOrigin'),
            '_wcdpi_packet_plus_international',
            '_wcdpi_packet_plus_international_section'
        );

        add_settings_field(
            '_wcdpi_packet_plus_international_service_level',
            __('Service level', 'woo-dp-internetmarke'),
            array($this, 'packetPlusInternationalServiceLevel'),
            '_wcdpi_packet_plus_international',
            '_wcdpi_packet_plus_international_section'
        );

        add_settings_field(
            '_wcdpi_packet_plus_international_terms_and_conditions',
            __('Terms and conditions', 'woo-dp-internetmarke'),
            array($this, 'packetPlusInternationalTermsAndConditions'),
            '_wcdpi_packet_plus_international',
            '_wcdpi_packet_plus_international_section'
        );
    }

    public function sanitizeSettings($settings)
    {
        if (key_exists('_wcdpi_packet_plus_international_activation', $settings)
            && $settings['_wcdpi_packet_plus_international_activation'] == 1
        ) {
            if (!$settings['_wcdpi_packet_plus_international_terms_and_conditions']) {
                add_settings_error(
                    '_wcdpi_settings_group_packet_plus_international',
                    'login_error',
                    __('You must accept terms and conditions to use this service.', 'woo-dp-internetmarke'),
                    'error'
                );

                return $this->settings;
            }

            if (!$settings['_wcdpi_packet_plus_international_ekp']) {
                add_settings_error(
                    '_wcdpi_settings_group_packet_plus_international',
                    'login_error',
                    __('You must enter your EKP number to start using this service.', 'woo-dp-internetmarke'),
                    'error'
                );

                return $this->settings;
            }
        }

        if (array_key_exists('packet_plus_international_logout', (array)$_POST)) {
            try {
                do_action('_wcdpi_packet_plus_international_logout');

                return array();
            } catch (Exception $e) {
                Internetmarke()->logger()->error($e->getMessage(), $e->getTrace());

                return array();
            }
        }

        if (!$settings['_wcdpi_packet_plus_international_ekp']) {
            $settings['_wcdpi_packet_plus_international_ekp'] = self::DEFAULT_EKP;
        }

        return $settings;
    }

    public function packetPlusInternationalHeader()
    {
        _e('Here you can enter your EKP number to enable Warenpost KT (Kilotarif) Harmonized Labels.', 'woo-dp-internetmarke');
    }

    public function packetPlusInternationalActivation()
    {
        $field    = '_wcdpi_packet_plus_international_activation';
        $loggedIn = UserCredentialsRepository::instance()->check();

        $html = sprintf(
            '<select id="%s" name="_wcdpi_settings_packet_plus_international[%s]" disabled="disabled">',
            $field,
            $field
        );

        $html .= sprintf(
            '<option value="1" %s>%s</option>',
            $loggedIn ? ' selected' : '',
            __('Yes', 'woo-dp-internetmarke')
        );

        $html .= sprintf(
            '<option value="0" %s>%s</option>',
            $loggedIn ? '' : ' selected',
            __('No', 'woo-dp-internetmarke')
        );

        $html .= '</select>';

        $html .= $this->createDescription(
            __('When the Portokasse service is activated, the Harmonized Label service is automatically activated.', 'woo-dp-internetmarke')
        );

        echo $html;
    }

    public function packetPlusInternationalEkp()
    {
        $field = '_wcdpi_packet_plus_international_ekp';
        $value = $this->getSettingValue($field);

        $html = $this->createInputField('_wcdpi_settings_packet_plus_international', $field, $value);

        $html .= $this->createDescription(
            __('Only applicable for Warenpost KT (Kilotarif) harmonized labels.', 'woo-dp-internetmarke')
        );

        echo $html;
    }

    public function packetPlusInternationalDefaultShipmentNatureType()
    {
        $field = '_wcdpi_packet_plus_international_default_shipment_nature_type';
        $value = $this->getSettingValue($field, false, '0');

        $html = sprintf(
            '<select id="%s" name="_wcdpi_settings_packet_plus_international[%s]">',
            $field,
            $field
        );

        $html .= sprintf(
            '<option value="%s" %s>%s</option>',
            'SALE_GOODS',
            ($value == 'SALE_GOODS') ? 'selected="selected"' : '',
            __('Sale goods', 'woo-dp-internetmarke')
        );

        $html .= sprintf(
            '<option value="%s" %s>%s</option>',
            'RETURN_GOODS',
            ($value == 'RETURN_GOODS') ? 'selected="selected"' : '',
            __('Return goods (Others)', 'woo-dp-internetmarke')
        );

        $html .= sprintf(
            '<option value="%s" %s>%s</option>',
            'GIFT',
            ($value == 'GIFT') ? 'selected="selected"' : '',
            __('Gift', 'woo-dp-internetmarke')
        );

        $html .= sprintf(
            '<option value="%s" %s>%s</option>',
            'COMMERCIAL_SAMPLE',
            ($value == 'COMMERCIAL_SAMPLE') ? 'selected="selected"' : '',
            __('Commercial sample', 'woo-dp-internetmarke')
        );

        $html .= sprintf(
            '<option value="%s" %s>%s</option>',
            'DOCUMENTS',
            ($value == 'DOCUMENTS') ? 'selected="selected"' : '',
            __('Documents', 'woo-dp-internetmarke')
        );

        $html .= sprintf(
            '<option value="%s" %s>%s</option>',
            'MIXED_CONTENTS',
            ($value == 'MIXED_CONTENTS') ? 'selected="selected"' : '',
            __('Mixed contents (Others)', 'woo-dp-internetmarke')
        );

        $html .= sprintf(
            '<option value="%s" %s>%s</option>',
            'OTHERS',
            ($value == 'OTHERS') ? 'selected="selected"' : '',
            __('Others', 'woo-dp-internetmarke')
        );

        $html .= '</select>';

        echo $html;
    }

    public function packetPlusInternationalDefaultLineItemOrigin()
    {
        $field = '_wcdpi_packet_plus_international_default_line_item_origin';
        $value = $this->getSettingValue($field, false, 'DE');

        $html = sprintf(
            '<select id="%s" name="_wcdpi_settings_packet_plus_international[%s]">',
            $field,
            $field
        );

        foreach (\WC()->countries->get_countries() as $iso2 => $name) {
            $html .= sprintf(
                '<option value="%s" %s>%s</option>',
                $iso2,
                ($value == $iso2) ? 'selected="selected"' : '',
                $name
            );
        }

        $html .= '</select>';

        echo $html;
    }

    public function packetPlusInternationalServiceLevel()
    {
        $field = '_wcdpi_packet_plus_international_service_level';
        $value = $this->getSettingValue($field, false, '0');

        $html = sprintf(
            '<select id="%s" name="_wcdpi_settings_packet_plus_international[%s]">',
            $field,
            $field
        );

        $html .= sprintf(
            '<option value="%s" %s>%s</option>',
            'PRIORITY',
            ($value == 'PRIORITY') ? 'selected="selected"' : '',
            __('Priority', 'woo-dp-internetmarke')
        );

        $html .= '</select>';

        echo $html;
    }

    public function packetPlusInternationalTermsAndConditions()
    {
        $field = '_wcdpi_packet_plus_international_terms_and_conditions';
        $value = $this->getSettingValue($field, true);

        $html = '<fieldset>';
        $html .= sprintf('<label for="%s">', $field);

        $html .= sprintf(
            '<input id="%s" type="checkbox" name="_wcdpi_settings_packet_plus_international[%s]" value="1" %s/>',
            $field,
            $field,
            checked(true, $value, false)
        );

        $html .= sprintf(
            __('Terms and conditions of the <b>Deutsche Post</b> (incl. <b>Packet Plus</b>) can be found <a href="%s" target="_blank">here</a>', 'woo-dp-internetmarke'),
            'https://www.deutschepost.de/'
        );

        $html .= '</label>';
        $html .= $this->createDescription(__('By ticking the boxes and clicking the button I confirm that I have read and consent to the terms listed above.', 'woo-dp-internetmarke'));
        $html .= '</fieldset>';

        echo $html;
    }
}