<?php

namespace DeutschePost\Internetmarke\Settings;

use DeutschePost\Internetmarke\Abstracts\Settings as AbstractSettings;
use DeutschePost\Internetmarke\Integrations\Host\OneClickForApp as OneClickForAppHostIntegration;
use DeutschePost\Internetmarke\Interfaces\CredentialsRepository as CredentialsRepositoryInterface;
use DeutschePost\Internetmarke\Models\Credentials\Partner as PartnerCredentialsModel;
use DeutschePost\Internetmarke\Models\Credentials\User as UserCredentialsModel;
use DeutschePost\Internetmarke\Repositories\Credentials\Partner as PartnerCredentialsRepository;
use DeutschePost\Internetmarke\Repositories\Credentials\User as UserCredentialsRepository;
use Exception;
use SoapFault;
use function DeutschePost\Internetmarke\Internetmarke;

defined('ABSPATH') || exit;

class Portokasse extends AbstractSettings
{
    /**
     * @var string
     */
    const PORTOKASSE_WEB_URL = 'https://portokasse.deutschepost.de/portokasse/#!/';

    /**
     * @var array
     */
    protected $settings;

    /**
     * @var UserCredentialsRepository
     */
    private $userCredentialsRepository;

    /**
     * @var OneClickForAppHostIntegration
     */
    private $oneClickForAppHostIntegration;

    /**
     * Portokasse constructor.
     *
     * @param OneClickForAppHostIntegration $oneClickForAppHostIntegration
     */
    public function __construct(OneClickForAppHostIntegration $oneClickForAppHostIntegration)
    {
        $this->userCredentialsRepository = UserCredentialsRepository::instance();
        $this->oneClickForAppHostIntegration = $oneClickForAppHostIntegration;
        $this->settings = get_option('_wcdpi_settings_portokasse', array());

        add_action('admin_init', array($this, 'init'));
    }

    /**
     * Registers the "Portokasse" setting group.
     *
     * @see https://developer.wordpress.org/reference/functions/register_setting/#parameters
     */
    public function init()
    {
        register_setting(
            '_wcdpi_settings_group_portokasse',
            '_wcdpi_settings_portokasse',
            array(
                'sanitize_callback' => array($this, 'sanitizeSettings')
            )
        );

        add_settings_section(
            '_wcdpi_portokasse_status_section',
            __('Service status', 'woo-dp-internetmarke'),
            null,
            '_wcdpi_portokasse'
        );

        add_settings_field(
            '_wcdpi_portokasse_activation',
            __('Enabled', 'woo-dp-internetmarke'),
            array($this, 'portokasseActivation'),
            '_wcdpi_portokasse',
            '_wcdpi_portokasse_status_section'
        );

        add_settings_section(
            '_wcdpi_portokasse_section',
            __('Account', 'woo-dp-internetmarke'),
            array($this, 'portokasseHeader'),
            '_wcdpi_portokasse'
        );

        add_settings_field(
            '_wcdpi_portokasse_email',
            __('Email', 'woo-dp-internetmarke'),
            array($this, 'portokasseEmail'),
            '_wcdpi_portokasse',
            '_wcdpi_portokasse_section'
        );
        add_settings_field(
            '_wcdpi_portokasse_password',
            __('Password', 'woo-dp-internetmarke'),
            array($this, 'portokassePassword'),
            '_wcdpi_portokasse',
            '_wcdpi_portokasse_section'
        );
        add_settings_field(
            '_wcdpi_portokasse_terms_and_conditions_deutsche_post',
            __('Terms and conditions', 'woo-dp-internetmarke'),
            array($this, 'termsAndConditions'),
            '_wcdpi_portokasse',
            '_wcdpi_portokasse_section'
        );

        if ($this->userCredentialsRepository->check()) {
            add_settings_field(
                '_wcdpi_portokasse_balance',
                __('Portokasse', 'woo-dp-internetmarke'),
                array($this, 'balance'),
                '_wcdpi_portokasse',
                '_wcdpi_portokasse_section'
            );
            add_settings_field(
                '_wcdpi_portokasse_wallet_topup',
                __('Wallet Top-Up', 'woo-dp-internetmarke'),
                array($this, 'walletTopUp'),
                '_wcdpi_portokasse',
                '_wcdpi_portokasse_section'
            );
        }
    }

    /**
     * Callback for settings sanitization.
     *
     * @param array $settings
     *
     * @return array
     */
    public function sanitizeSettings($settings)
    {
        $partnerCredentialsRepository = PartnerCredentialsRepository::instance();

        // Portokasse logout
        if ($this->userCredentialsRepository->check() && !$settings['_wcdpi_portokasse_activation']) {
            try {
                $this->userCredentialsRepository->delete();

                return array();
            } catch (Exception $e) {
                Internetmarke()->logger()->error($e->getMessage(), $e->getTrace());
                add_settings_error(
                    '_wcdpi_settings_group_portokasse',
                    'login_error',
                    $e->getMessage(),
                    'error'
                );

                return array();
            }
        }

        if (empty($settings['_wcdpi_portokasse_terms_and_conditions_stina'])) {
            $settings['_wcdpi_portokasse_terms_and_conditions_stina']
                = $this->settings['_wcdpi_portokasse_terms_and_conditions_stina'];
        }

        if (empty($settings['_wcdpi_portokasse_terms_and_conditions_deutsche_post'])) {
            $settings['_wcdpi_portokasse_terms_and_conditions_deutsche_post']
                = $this->settings['_wcdpi_portokasse_terms_and_conditions_deutsche_post'];
        }

        if (!$settings['_wcdpi_portokasse_terms_and_conditions_stina'] ||
            !$settings['_wcdpi_portokasse_terms_and_conditions_deutsche_post']
        ) {
            add_settings_error(
                '_wcdpi_settings_group_portokasse',
                'login_error',
                __('You must accept both terms and conditions to use the plugin.', 'woo-dp-internetmarke'),
                'error'
            );

            return array();
        }

        if (empty($settings['_wcdpi_portokasse_email'])) {
            $settings['_wcdpi_portokasse_email'] = $this->settings['_wcdpi_portokasse_email'];
        }

        if (empty($settings['_wcdpi_portokasse_password'])) {
            $settings['_wcdpi_portokasse_password'] = $this->settings['_wcdpi_portokasse_password'];
        }

        $email    = sanitize_email($settings['_wcdpi_portokasse_email']);
        $password = sanitize_text_field($settings['_wcdpi_portokasse_password']);

        try {
            // Update the partner credentials
            $partnerCredentials = apply_filters('wcdpi_get_partner_credentials', null);
            apply_filters('wcdpi_internetmarke_activate_plugin', null);
            //update_option('_wcdpi_partner_credentials', $partnerCredentials);
            $partnerCredentialsRepository->set($partnerCredentials);

            /** @var PartnerCredentialsModel $partnerCredentialsModel */
            $partnerCredentialsModel = $partnerCredentialsRepository->get();

            $partnerId      = $partnerCredentialsModel->get('OneClickForApplication', 'PartnerId');
            $keyPhase       = $partnerCredentialsModel->get('OneClickForApplication', 'KeyPhase');
            $marketplaceKey = $partnerCredentialsModel->get('OneClickForApplication', 'MarketplaceKey');

            $this->oneClickForAppHostIntegration->setPartnerId($partnerId);
            $this->oneClickForAppHostIntegration->setKeyPhase($keyPhase);
            $this->oneClickForAppHostIntegration->setMarketplaceKey($marketplaceKey);

            apply_filters('wcdpi_integration_oneclickforapp_login', $email, $password);

            // Set the user credentials
            $this->userCredentialsRepository->set(
                array(
                    'email' => $email,
                    'password' => $password
                )
            );

            // Update the settings
            return array(
                '_wcdpi_portokasse_email'                              => $email,
                '_wcdpi_portokasse_password'                           => $password,
                '_wcdpi_portokasse_terms_and_conditions_stina'         => 1,
                '_wcdpi_portokasse_terms_and_conditions_deutsche_post' => 1
            );
        } catch (SoapFault $sf) {
            Internetmarke()->logger()->error($sf->getMessage(), $sf->getTrace());
            add_settings_error(
                '_wcdpi_settings_group_portokasse',
                'login_error',
                $sf->getMessage(),
                'error'
            );
            $this->userCredentialsRepository->delete();

            return array();
        } catch (Exception $e) {
            Internetmarke()->logger()->error($e->getMessage(), $e->getTrace());
            add_settings_error(
                '_wcdpi_settings_group_portokasse',
                'login_error',
                $e->getMessage(),
                'error'
            );
            $this->userCredentialsRepository->delete();

            return array();
        }
    }

    public function portokasseHeader()
    {
        if ($this->userCredentialsRepository->check()) {
            _e(
                'Here you can top-up your wallet. Please note that the minimum amount is 10,00 € and that you will be redirected to the Plugin website to finish the transaction.',
                'woo-dp-internetmarke'
            );
        }

        else {
            _e(
                sprintf(
                    'Your Portokasse account credentials. The same one used at the <a href="%s" target="_blank">web shop</a>.',
                    self::PORTOKASSE_WEB_URL
                ),
                'woo-dp-internetmarke'
            );
        }
    }

    public function portokasseActivation()
    {
        $field = '_wcdpi_portokasse_activation';

        $html = sprintf(
            '<select id="%s" name="_wcdpi_settings_portokasse[%s]" %s>',
            $field,
            $field,
            $this->userCredentialsRepository->check() ? '' : ' disabled="disabled"'
        );

        $html .= sprintf(
            '<option value="1" %s>%s</option>',
            $this->userCredentialsRepository->check() ? ' selected="selected"' : '',
            __('Yes', 'woo-dp-internetmarke')
        );

        $html .= sprintf(
            '<option value="0" %s>%s</option>',
            $this->userCredentialsRepository->check() ? '' : ' selected="selected"',
            __('No', 'woo-dp-internetmarke')
        );

        $html .= '</select>';

        if (!$this->userCredentialsRepository->check()) {
            $html .= $this->createDescription(__('When you are logged in, the service is automatically enabled.', 'woo-dp-internetmarke'));
        }

        echo $html;
    }

    public function portokasseEmail()
    {
        $field = '_wcdpi_portokasse_email';
        $value = $this->getSettingValue($field);

        $html = $this->createInputField(
            '_wcdpi_settings_portokasse',
            $field,
            $value,
            'text',
            $this->userCredentialsRepository->check()
        );

        $html .= $this->createDescription(__('Your Deutsche Post Portokasse Account Email', 'woo-dp-internetmarke'));

        echo $html;
    }

    public function portokassePassword()
    {
        $field = '_wcdpi_portokasse_password';
        $value = $this->getSettingValue($field);

        $html = $this->createInputField(
            '_wcdpi_settings_portokasse',
            $field,
            $value,
            'password',
            $this->userCredentialsRepository->check()
        );

        $html .= $this->createDescription(__('Your Deutsche Post Portokasse Account Password', 'woo-dp-internetmarke'));

        echo $html;
    }

    public function balance()
    {
        $html = sprintf(
            '<p><span id="wcdpi-portokasse-amount" style="margin-right: 10px;">%s</span> <a id="wcdpi-update-portokasse-wallet" class="button button-secondary">%s</a> <img id="wcdpi-update-portokasse-spinner" src="/wp-admin/images/spinner.gif"></p>',
            wcdpi_format_price(get_option('wcdpi_oneclickforapp_wallet_balance') / 100),
            __('Update', 'woo-dp-internetmarke')
        );

        echo $html;
    }

    public function termsAndConditions()
    {
        $fieldStina        = '_wcdpi_portokasse_terms_and_conditions_stina';
        $fieldDeutschePost = '_wcdpi_portokasse_terms_and_conditions_deutsche_post';
        $valueStina        = $this->getSettingValue($fieldStina, true);
        $valueDeutschePost = $this->getSettingValue($fieldDeutschePost, true);

        // STINA
        $html = '<fieldset>';
        $html .= sprintf('<label for="%s">', $fieldStina);

        $html .= sprintf(
            '<input %s id="%s" type="checkbox" name="_wcdpi_settings_portokasse[%s]" value="1" %s/>',
            $this->userCredentialsRepository->check() ? ' disabled' : '',
            $fieldStina,
            $fieldStina,
            checked(true, $valueStina, false)
        );

        $html .= sprintf(
            __('Terms and conditions of <b>STINA Business Solutions GmbH</b> or its affiliates can be found <a href="%s" target="_blank">here</a>', 'woo-dp-internetmarke'),
            'https://www.stina-global.com/information/terms-and-conditions/'
        );

        $html .= '</label>';
        $html .= '</fieldset>';

        // Deutsche Post
        $html .= '<fieldset>';
        $html .= sprintf('<label for="%s">', $fieldDeutschePost);

        $html .= sprintf(
            '<input %s id="%s" type="checkbox" name="_wcdpi_settings_portokasse[%s]" value="1" %s/>',
            $this->userCredentialsRepository->check() ? ' disabled' : '',
            $fieldDeutschePost,
            $fieldDeutschePost,
            checked(true, $valueDeutschePost, false)
        );

        $html .= sprintf(
            __('Terms and conditions of the <b>Deutsche Post</b> (incl. <b>Deutsche Post Warenpost</b>, <b>Portokasse</b> and <b>Plugin</b>) can be found <a href="%s" target="_blank">here</a>', 'woo-dp-internetmarke'),
            'https://www.deutschepost.de/'
        );

        $html .= '</label>';
        $html .= $this->createDescription(
            __('By ticking the boxes and clicking the button I confirm that I have read and consent to the terms listed above.', 'woo-dp-internetmarke')
        );
        $html .= '</fieldset>';

        echo $html;
    }

    public function logout()
    {
        $html = sprintf(
            '<button type="submit" name="portokasse_logout">%s</button>',
            __('Logout', 'woo-dp-internetmarke')
        );

        $html .= $this->createDescription(__('A logout resets all settings on this page.', 'woo-dp-internetmarke'));

        echo $html;
    }

    public function walletTopUp()
    {
        /** @var UserCredentialsModel $userCredentials */
        $userCredentials = $this->userCredentialsRepository->get();

        $fieldWalletTopUp = '_wcdpi_portokasse_wallet_topup';
        $userToken        = '';
        $walletBalance    = '';

        if ($userCredentials->getEmail()) {
            $userToken = $this->oneClickForAppHostIntegration->getUserToken();
            $walletBalance = $this->oneClickForAppHostIntegration->getWalletBalance();
        }

        $html = '<p>';

        $html .= sprintf(
            '<input id="%s" type="text" class="regular-text" name="_wcdpi_settings_portokasse[%s]" placeholder="%s" style="margin-right: 10px;"/>',
            $fieldWalletTopUp,
            $fieldWalletTopUp,
            __('Minimum amount: 10,00€', 'woo-dp-internetmarke')
        );

        $html .= sprintf(
            '<a class="button button-secondary"
                       id="wcdpi-portokasse-wallet-topup-button"
                       data-portokasse-url="%s"
                       data-success-url="%s"
                       data-cancel-url="%s"
                       data-partner-id="%s"
                       data-key-phase="%s"
                       data-user-token="%s"
                       data-schluessel-dpwn-partner="%s"
                       data-wallet="%d">%s</a>',
            WCDPI_WALLET_TOP_UP_URL,
            admin_url('admin.php?page=internetmarke&wallet=success'),
            admin_url('admin.php?page=internetmarke&wallet=error'),
            $this->oneClickForAppHostIntegration->getPartnerID(),
            $this->oneClickForAppHostIntegration->getKeyPhase(),
            $userToken,
            $this->oneClickForAppHostIntegration->getMarketplaceKey(),
            $walletBalance,
            __('Wallet Top-Up', 'woo-dp-internetmarke')
        );

        $html .= '</p>';

        echo $html;
    }
}