<?php

namespace DeutschePost\Internetmarke;

use Exception;
use DeutschePost\Internetmarke\Connectors\REST\FORKLane;
use WP_Roles;
use function time;

defined('ABSPATH') || exit;

class Setup
{
    /**
     * Current database version.
     */
    const PLUGIN_DB_VERSION = 302;

    /**
     * Activation script.
     */
    public static function activate()
    {
        self::requirements();
        self::migrate('ACTIVATE');
        self::scheduleCron();
        self::initSettings();
        self::createRoles();

        update_option('internetmarke_db_version', self::PLUGIN_DB_VERSION, true);

        try {
            $forkLane = new FORKLane();
            apply_filters('wcdpi_internetmarke_activate_plugin', null);
            unset($forkLane);
        } catch (Exception $e) {
        }
    }

    /**
     * Deactivation script.
     */
    public static function deactivate()
    {
        self::migrate('DEACTIVATE');
        self::unscheduleCron();
        self::deactivateOptions();
        self::deleteCookies();

        try {
            apply_filters('wcdpi_internetmarke_deactivate_plugin', null);
        } catch (Exception $e) {
        }
    }

    /**
     * Uninstallation script.
     */
    public static function uninstall()
    {
        self::migrate('UNINSTALL');
        self::unscheduleCron();
        self::uninstallOptions();
        self::deleteCookies();
        self::removeRoles();
    }

    /**
     * Performs checks to see if the server meets the plugin requirements.
     */
    public static function requirements()
    {
        if (!extension_loaded('soap') || !class_exists('SOAPClient')) {
            $message = __(
                'Please install the PHP SOAP extension to start using the Deutsche Post Internetmarke plugin.',
                'woo-dp-internetmarke'
            );

            exit($message);
        }
    }

    /**
     * Update script.
     */
    public static function update()
    {
        $dbVersion = get_option('internetmarke_db_version');

        if (!$dbVersion) {
            self::migrate('ACTIVATE');
            update_option('internetmarke_db_version', self::PLUGIN_DB_VERSION, true);
        } elseif ($dbVersion < self::PLUGIN_DB_VERSION) {
            self::migrate('UPGRADE');
            update_option('internetmarke_db_version', self::PLUGIN_DB_VERSION, true);
        }
    }

    /**
     * Execute migration files.
     *
     * @param string $migrationMode
     */
    public static function migrate($migrationMode)
    {
        global $wpdb;

        $dbVersion = get_option('internetmarke_db_version');
        $regex     = "/(\d{14})_(.*?)_(.*?)\.php/";

        foreach (glob(WCDPI_MIGRATIONS_DIR . "*.php") as $filename) {
            if (preg_match($regex, basename($filename), $match)) {
                $fileBasename    = $match[0];
                $fileDatetime    = $match[1];
                $fileVersion     = $match[2];
                $fileDescription = $match[3];

                if (intval($fileVersion) <= self::PLUGIN_DB_VERSION && intval($fileVersion) > $dbVersion) {
                    require_once $filename;
                }
            }
        }
    }

    /**
     * Schedules the Plugin Cron job.
     */
    public static function scheduleCron()
    {
        // Product list update
        if (!wp_next_scheduled('internetmarke_update_product_list')) {
            wp_schedule_event(time(), 'daily', 'internetmarke_update_product_list');
        }

        // Wallet update
        if (!wp_next_scheduled('internetmarke_update_portokasse_wallet')) {
            wp_schedule_event(time(), 'daily', 'internetmarke_update_portokasse_wallet');
        }
    }

    /**
     * Unschedules the Plugin Cron job.
     */
    public static function unscheduleCron()
    {
        // Product list update
        wp_unschedule_event(
            wp_next_scheduled('internetmarke_update_product_list'),
            'internetmarke_update_product_list'
        );

        // Wallet update
        wp_unschedule_event(
            wp_next_scheduled('internetmarke_update_portokasse_wallet'),
            'internetmarke_update_portokasse_wallet'
        );
    }

    /**
     * Unschedules the Plugin Cron job.
     */
    public static function initSettings()
    {
        if (!get_option('_wcdpi_settings_general', false)) {
            update_option(
                '_wcdpi_settings_general',
                array(
                    '_wcdpi_label_options_shipping_list'       => 0,
                    '_wcdpi_default_sender_first_name'         => '',
                    '_wcdpi_default_sender_last_name'          => '',
                    '_wcdpi_default_sender_company_name'       => '',
                    '_wcdpi_default_sender_street'             => '',
                    '_wcdpi_default_sender_house_number'       => '',
                    '_wcdpi_default_sender_additional'         => '',
                    '_wcdpi_default_sender_zip'                => '',
                    '_wcdpi_default_sender_city'               => '',
                    '_wcdpi_default_sender_country'            => '',
                    '_wcdpi_page_formats_default_label_format' => 1,
                    '_wcdpi_favorite_labels_selection'         => array()
                )
            );
        }

        if (!get_option('_wcdpi_settings_portokasse', false)) {
            update_option('_wcdpi_settings_portokasse', array());
        }

        if (!get_option('_wcdpi_settings_internetmarke_1c4a', false)) {
            update_option('_wcdpi_settings_internetmarke_1c4a', array());
        }

        if (!get_option('_wcdpi_settings_tools', false)) {
            update_option('_wcdpi_settings_tools', array());
        }

        if (!get_option('_wcdpi_settings_logger', false)) {
            update_option('_wcdpi_settings_logger', array());
        }

        if (!get_option('_wcdpi_settings_port_checker', false)) {
            update_option('_wcdpi_settings_port_checker', array());
        }

        if (!get_option('_wcdpi_settings_packet_plus_international', false)) {
            update_option(
                '_wcdpi_settings_packet_plus_international',
                array(
                    '_wcdpi_packet_plus_international_activation' => 1,
                    '_wcdpi_packet_plus_international_default_shipment_nature_type' => 'SALE_GOODS',
                    '_wcdpi_packet_plus_international_default_line_item_origin' => 'DE',
                    '_wcdpi_packet_plus_international_service_level' => 'PRIORITY'
                )
            );
        }

        if (!get_option('_wcdpi_settings_packet_plus_cross_border', false)) {
            update_option('_wcdpi_settings_packet_plus_cross_border', array());
        }
    }

    /**
     * Add Deutsche Post Internetmarke roles
     */
    public static function createRoles()
    {
        global $wp_roles;

        if (!class_exists('\WP_Roles')) {
            return;
        }

        if (!isset($wp_roles)) {
            $wp_roles = new WP_Roles();
        }

        // Dummy gettext calls to get strings in the catalog.
        /* translators: user role */
        _x('Internetmarke manager', 'User role', 'woo-dp-internetmarke');

        // Internetmarke manager role.
        add_role(
            'internetmarke_manager',
            'Internetmarke manager',
            array(
                'level_9'                => true,
                'level_8'                => true,
                'level_7'                => true,
                'level_6'                => true,
                'level_5'                => true,
                'level_4'                => true,
                'level_3'                => true,
                'level_2'                => true,
                'level_1'                => true,
                'level_0'                => true,
                'read'                   => true,
                'read_private_pages'     => true,
                'read_private_posts'     => true,
                'edit_posts'             => true,
                'edit_pages'             => true,
                'edit_published_posts'   => true,
                'edit_published_pages'   => true,
                'edit_private_pages'     => true,
                'edit_private_posts'     => true,
                'edit_others_posts'      => true,
                'edit_others_pages'      => true,
                'publish_posts'          => true,
                'publish_pages'          => true,
                'delete_posts'           => true,
                'delete_pages'           => true,
                'delete_private_pages'   => true,
                'delete_private_posts'   => true,
                'delete_published_pages' => true,
                'delete_published_posts' => true,
                'delete_others_posts'    => true,
                'delete_others_pages'    => true,
                'manage_categories'      => true,
                'manage_links'           => true,
                'moderate_comments'      => true,
                'upload_files'           => true,
                'export'                 => true,
                'import'                 => true,
                'list_users'             => true,
                'edit_theme_options'     => true,
            )
        );

        foreach (self::getCoreCapabilities() as $capGroup) {
            foreach ($capGroup as $cap) {
                $wp_roles->add_cap('internetmarke_manager', $cap);
                $wp_roles->add_cap('administrator', $cap);
            }
        }
    }

    /**
     * Remove Deutsche Post Internetmarke roles
     */
    public static function removeRoles()
    {
        global $wp_roles;

        if (!class_exists('\WP_Roles')) {
            return;
        }

        if (!isset($wp_roles)) {
            $wp_roles = new WP_Roles();
        }

        foreach (self::getCoreCapabilities() as $capGroup) {
            foreach ($capGroup as $cap) {
                $wp_roles->remove_cap('internetmarke_manager', $cap);
                $wp_roles->remove_cap('administrator', $cap);
            }
        }

        remove_role('internetmarke_manager');
    }

    /**
     * Returns the plugin's core capabilities.
     *
     * @return array
     */
    public static function getCoreCapabilities()
    {
        $capabilities = array();

        $capabilities['core'] = array(
            'manage_internetmarke'
        );

        $capabilities['shipment'] = array(
            'prepare_shipment',
            'create_shipment',
            'delete_shipment',
            'edit_shipment',
            'read_shipment',
            'prepare_shipments',
            'create_shipments',
            'delete_shipments',
            'edit_shipments',
            'read_shipments'
        );

        $capabilities['label'] = array(
            'preview_label'
        );

        return $capabilities;
    }

    /**
     * Removes the plugin's options when completely uninstalling.
     */
    public static function uninstallOptions()
    {
        delete_option('wcdpi_password_crypto_args');
        delete_option('wcdpi_default_address');
        delete_option('wcdpi_favorites');
        delete_option('wcdpi_partner');
        delete_option('wcdpi_page_format_id');
        delete_option('internetmarke_db_version');

        // New Settings (since v2.0)
        delete_option('_wcdpi_partner_credentials');
        delete_option('_wcdpi_crypto_args');
        delete_option('_wcdpi_user_credentials');
        delete_option('_wcdpi_settings_general');
        delete_option('_wcdpi_settings_portokasse');
        delete_option('_wcdpi_settings_internetmarke_1c4a');
        delete_option('_wcdpi_settings_tools');
        delete_option('_wcdpi_settings_logger');
        delete_option('_wcdpi_settings_port_checker');
        delete_option('_wcdpi_settings_packet_plus_international');
        delete_option('_wcdpi_settings_packet_plus_cross_border');
    }

    public static function deactivateOptions()
    {
        delete_option('_wcdpi_user_credentials');
        delete_option('_wcdpi_settings_portokasse');
        delete_option('_wcdpi_settings_packet_plus_international');
        delete_option('_wcdpi_settings_packet_plus_cross_border');
    }

    public static function deleteCookies()
    {
        setcookie('internetmarke_recipient_address', '', time() - 3600);
        setcookie('internetmarke_sender_address', '', time() - 3600);
    }
}